# -*- coding: utf-8 -*-
"""
     Rar file operations module
"""

import patoolib

class RarHelper:
    """Class for easing the Rar file operations"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """

    def ExtractRarFile(self, RarFileName, destPath):
        """Rar file extraction procedure

        Args:
            self: The reserved object 'self'
            RarFileName: Extractable file path + name
            destPath: Destination path for extracted files
        """
        patoolib.extract_archive(RarFileName, outdir=destPath)



# -*- coding: utf-8 -*-
"""
     Data synchronization module (synchronizes data between tables)
"""

class GDPSyncroniserHelper(object):
    """ESRI table synchronization class"""

    def __init__(self, gp, Tool=None):
        """Class initialization operation

        Args:
            self: The reserved object 'self'
            gp: ArcPy GP object
        """
        self.gp = gp
        if Tool != None:
            self.isTool = True
        else:
            self.isTool = False
        self.Tool = Tool

    def AddWarning(self, str):
        """Wrapper for the 'AddWarning' procedure

        Args:
            self: The reserved object 'self'
            str: Output string
        """
        if self.isTool:
            self.Tool.AddWarning(str)
        else:
            self.gp.AddWarning(str)

    def AddMessage(self, str):
        """Wrapper for the 'AddMessage' procedure

        Args:
            self: The reserved object 'self'
            str: Output string
        """
        if self.isTool:
            self.Tool.AddMessage(str)
        else:
            self.gp.AddMessage(str)

    def DoSync(self, definition, workspace=None, startEditing=False, startOperation=False, with_undo=False, multiuser=False):
        """Data synchronization procedure

        Args:
            self: The reserved object 'self'
            definition: 'SyncDefinition' object which describes the synchronization parameters
            workspace: DB in which to start data editing
            startEditing: Start the edit session in the DB specified in 'workspace' parameter? (Default = False)
            startOperation: Start the edit operation in the DB specified in 'workspace' parameter? (Default = False)
            with_undo: Sets whether the undo and redo stacks are enabled or disabled for an edit session. (Default = False)
            multiuser: Sets whether a DB contains a nonversioned, or versioned dataset. (Default = False)

        Returns:
            * output - Returns the report about the process execution
            * outputErrors - Returns the error description
            * errIDs - list of IDs that had an error
            * SyncIDs - list of IDs that was syncronized
        """
        outputErrors = u""
        output = u""
        errIDs = []
        SyncIDs = []

        if definition.inTableQuery != None:
            hasInQuery = True
        else:
            hasInQuery = False

        inFields = ()
        inFields = (definition.inTableJoinField, ) + definition.inTableFields
        outFields = ()
        outFields = (definition.outTableJoinField, ) + definition.outTableFields

        self.AddMessage(u'>>>>Start the synchronization of the tables {0} and {1}'.format(definition.inTable, definition.outTable))
        self.AddMessage(u'>>>>>>>>Join {0}.{1}={2}.{3}'.format(definition.inTable, definition.inTableJoinField, definition.outTable, definition.outTableJoinField))
        self.AddMessage(u'>>>>>>>>Input fields: [{0}]'.format(", ".join(definition.inTableFields)))
        if hasInQuery:
            self.AddMessage(u'>>>>>>>>Input query: [{0}]'.format(definition.inTableQuery))
        self.AddMessage(u'>>>>>>>>Output fields: [{0}]'.format(", ".join(definition.outTableFields)))

        if startEditing or startOperation:
            edit = self.gp.da.Editor(workspace)
        if startEditing:
            edit.startEditing(with_undo, multiuser)
        if startOperation:
            edit.startOperation()

        with self.gp.da.SearchCursor(definition.inTable, inFields, definition.inTableQuery) as cur:
            i = 0
            j = 0
            k = 0
            err = 0
            for row in cur:
                output = self.__DoSyncRow(row, definition.outTable, outFields, definition.outTableJoinField, definition.messageDefinition, definition.idvalueseparator, definition.createNew)
                if type(output) is int:
                    i = i + output
                    SyncIDs.append(row[0])
                else:
                    if self.isTool:
                        self.AddMessage(u'        ...{0}'.format(output))
                        outputErrors = outputErrors + u'{0}\n'.format(output)
                        err = err + 1
                        errIDs.append(row[0])
                j = j + 1
                k = k + 1

                if k >= 1000:
                    if self.isTool:
                        self.AddMessage(u'>>>> ...Processed [{0}], stored [{1}], faulty [{2}]'.format(j, i, err))
                    k = 0
                    if startOperation:
                        edit.stopOperation()
                    if startOperation:
                        edit.startOperation()

        if startOperation:
            edit.stopOperation()
        if startEditing:
            edit.stopEditing(True)

        if self.isTool:
            self.AddMessage(u'>>>> ...Processed [{0}], stored [{1}], faulty [{2}]'.format(j, i, err))
            output = u'Processed [{0}], stored [{1}], faulty [{2}]'.format(j, i, err)

        return output, outputErrors, errIDs, SyncIDs

    def __DoSyncRow(self, inRow, outTable, outTableFields, outTableJoinField, messageString, idvalueseparator, createNew):
        """Procedure performs row synchronization

        Args:
            self: The reserved object 'self'
            inRow: Row to synchronize
            outTable: Output table
            outTableFields: Output table fields
            outTableJoinField: Output table join field
            messageString: Output message formatting
            createNew: Create new record if needed

        Returns:
            * output - 0, If no changes were necessary; 1, If there were any changes
            * Error description (in case there were errors)
        """
        if inRow[0] == None:
            return u"Error: join field is empty " + messageString.format(*inRow)

        with self.gp.da.UpdateCursor(outTable, outTableFields, u'{0} = {2}{1}{2}'.format(outTableJoinField, inRow[0], idvalueseparator)) as cur:
            output = 0
            i = 0
            foundRow = False
            for row in cur:
                if i == 0:
                    foundRow = True
                    output, row = self.__DoSyncFields(inRow, row)
                    if output == 1:
                        cur.updateRow(row)
                    i = i+1
                else:
                    return u"Error: row in the results table is not unique" + messageString.format(*inRow)

            if not foundRow:
                if createNew:
                    with self.gp.da.InsertCursor(outTable, outTableFields) as insCur:
                        newRow = [None for _ in range(len(inRow))]
                        output, newRow = self.__DoSyncFields(inRow, newRow)
                        insCur.insertRow(newRow)
                else:
                    return u"Error: no related records found in the results table " + messageString.format(*inRow)

            return output

    def __DoSyncFields(self, inRow, outRow):
        """Procedure performs the field synchronization

        Args:
            self: The reserved object 'self'
            inRow: Row to synchronize
            outRow: Row to which synchronize

        Returns:
            * output - 0, If no changes were necessary; 1, If there were any changes
            * outRow - Altered row
        """
        output = 0
        for x in range(0, len(inRow)):
            if inRow[x] != outRow[x]:
                outRow[x] = inRow[x]
                output = 1
        return output, outRow

class SyncDefinition(object):
    """Synchronization definition description class"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """
        self.inTable = ""
        self.inTableFields = ()
        self.inTableQuery = None
        self.inTableJoinField = ""
        self.outTable = ""
        self.outTableFields = ()
        self.outTableJoinField = ""
        self.messageDefinition = ""
        self.idvalueseparator = ""
        self.createNew = False



# -*- coding: utf-8 -*-
"""
     Module for xml parameter file procedures
"""

import os
import codecs
from lxml import etree
from collections import OrderedDict

utf8_parser = etree.XMLParser(encoding='utf-8')

class XMLParams(object):
    """xml parameter reading support class"""
    def __init__(self, Tool, ConfigFolder, ConfigFile):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            Tool: GISPythonTool (Optional)
            ConfigFolder: Configuration file storing directory
            ConfigFile: Name of the configuration file (without extension)
        """
        self.Tool = Tool
        self.ConfigFolder = ConfigFolder
        self.ConfigFile = ConfigFile
        if not Tool == None:
            self.ConfigPath = os.path.join(self.Tool.ExecutePatch, ConfigFolder, ConfigFile)
        else:
            if not ConfigFolder == None:
                self.ConfigPath = os.path.join(ConfigFolder, ConfigFile)
            else:
                self.ConfigPath = ConfigFile
        self.Params = []

    def GetParams(self):
        """Get parameters from the parameter file

        Args:
            self: The reserved object 'self'
        """
        with open(self.ConfigPath, 'r') as xmlfile:
            xmllines = xmlfile.readlines()
        doc = etree.ElementTree(etree.XML("".join(xmllines)))
        self.Params = doc
        return self.Params

    def WriteParams(self):
        """Save parameters in the parameter file

        Args:
            self: The reserved object 'self'
        """
        with open(self.ConfigPath, 'w') as xmlfile:
            # xmlString = etree.tostring(self.Params, pretty_print=True, method="xml", xml_declaration=True, encoding="utf-8")
            self.Params.write(xmlfile, pretty_print=True, method="xml", xml_declaration=True, encoding="utf-8")


    def UpdateValueByPath(self, path, Value, index = 0, isString = False):
        elem = self.Params
        if isString:
            elem.xpath(path)[index].append(etree.fromstring(Value))
        else:
            elem.xpath(path)[index].text = Value

    def AppendValueByPath(self, path, key, Value, attrib, index = 0, isString = False):
        elem = self.Params
        elem = elem.xpath(path)[index]
        if key==None:
            node = elem
        else:
            node = etree.SubElement(elem, key, attrib)
        if isString:
            node.append(etree.fromstring(Value))
        else:
            node.text = Value

    def UpdateAtributeByPath(self, path, atribute, Value, index = 0):
        elem = self.Params
        elem.xpath(path)[index].attrib[atribute] = Value

    def GetValueByPath(self, path):
        elem = self.Params
        elem = elem.xpath(path)[0].text
        return elem

    def GetAtributeByPath(self, path, atribute):
        elem = self.Params
        elem = elem.xpath(path)[0].attrib[atribute]
        return elem


# -*- coding: utf-8 -*-
"""
     Error module
"""

class MyError(Exception):
    """Class for storing an error object"""
    def __init__(self, strerror):
        self.strerror = strerror
    def __str__(self):
        try: # string
            return str(self.strerror)
        except Exception:
            pass
            
        try: # unicode
            value = unicode(self.strerror)
            return value.encode("ascii", "backslashreplace")
        except Exception:
            pass
            
        try: # repr
            value = self.strerror.repr()
            return value
        except Exception:
            pass
            
        return '<unprintable %s object>' % type(value).__name__
    def __repr__(self):
        return self.__str__()
    def __unicode__(self):
        return self.strerror



# -*- coding: utf-8 -*-
"""
     Module for the package definition
"""

import os
import SysGISToolsSysParams

if __name__ == "__main__":
    """
        Execution of the module
    """
    print(u"""
    Hello!

    This is GISPython for ArcGIS 10.3.1.

    Version: """ + SysGISToolsSysParams.Version + u"""

    Author: LVM GEO

    Platform contains following additional modules:
    """)
    # Get the list of all python modules in GISPython folder
    filenames = [f for f in os.listdir(os.path.dirname(os.path.abspath(__file__))) if (os.path.isfile(os.path.join(os.path.dirname(os.path.abspath(__file__)), f)) and not f[0:3].upper() == "SYS" and f[-3:].upper() == '.PY' and f.upper() != 'MYERROR.PY' and f.upper() != 'GISPYTHONTOOL.PY' and f.upper() != 'GISPYTHONMODULE.PY' and f.upper() != '__INIT__.PY')]
    for fl in filenames:
            print('             ' + fl)



# -*- coding: utf-8 -*-
"""
     Zip file operations module
"""
import os
import zipfile

class ZipHelper:
    """Class for easing the Zip file operations"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """

    def CompressFile(self, filePath, zipFileName):
        """Compress file

        Args:
            self: The reserved object 'self'
            filePath: Archivable file path + name
            zipFileName: New Zip file path + name
        """
        zfile = zipfile.ZipFile(zipFileName, "w", zipfile.ZIP_DEFLATED)
        zfile.write(filePath, arcname=zfile.write(filePath, arcname=os.path.basename(filePath)))
        zfile.close()

    def CompressFileList(self, filePathList, zipFileName, base_dir=None, append=False):
        """Zip all files in the list

        Args:
            self: The reserved object 'self'
            filePathList: List of archivable file paths + names
            zipFileName: New Zip file path + name
            base_dir: dase dir to strip from paths in file lists
            append (Bool): (Optional) True if Zip exists and data is appended to it
        """
        if append:
            zfile = zipfile.ZipFile(zipFileName, 'a', zipfile.ZIP_DEFLATED, allowZip64=True)
        else:
            zfile = zipfile.ZipFile(zipFileName, 'w', zipfile.ZIP_DEFLATED, allowZip64=True)
        for filePath in filePathList:
            if base_dir is None:
                arc_name = os.path.basename(filePath)
            else:
                arc_name = _ireplace(base_dir, '', filePath)
            zfile.write(filePath, arcname=arc_name)
        zfile.close()

    def CompressDir(self, dirPath, zipFileName, excludeExt=[], append=False, start_path_in_zip=''):
        """Zip all files in the directory

        Args:
            self: The reserved object 'self'
            zipFileName (string): New Zip file path + name
            dirPath (string): Directory which contains archivable files
            excludeExt (list): File extensions not to include in the archive
            append (Bool): (Optional) True if Zip exists and data is appended to it
            start_path_in_zip (string): (Optional) used to specify subfolder in zipfile in with data vill be written
        """
        if append:
            zfile = zipfile.ZipFile(zipFileName, 'a', zipfile.ZIP_DEFLATED, allowZip64=True)
        else:
            zfile = zipfile.ZipFile(zipFileName, 'w', zipfile.ZIP_DEFLATED, allowZip64=True)
        for root, dirs, files in os.walk(dirPath):
            for file_path in files:
                do_compress = True
                for ext in excludeExt:
                    if file_path.endswith(ext):
                        do_compress = False
                if do_compress:
                    zfile.write(os.path.join(root, file_path), arcname=os.path.join(root, file_path).replace(dirPath, start_path_in_zip))
        zfile.close()

    def ExtractZipFile(self, zipFileName, destPath):
        """Extracts the compressed file

        Args:
            self: The reserved object 'self'
            zipFileName: Extractable file full path + name
            destPath: Destination path in which to extract the files
        """
        zfile = zipfile.ZipFile(zipFileName)
        zfile.extractall(destPath)
        zfile.close()

def _ireplace(old, new, text):
    idx = 0
    while idx < len(text):
        index_l = text.lower().find(old.lower(), idx)
        if index_l == -1:
            return text
        text = text[:index_l] + new + text[index_l + len(old):]
        idx = index_l + len(new)
    return text



# -*- coding: utf-8 -*-
"""
     Deployment publishing operations module
"""

import codecs
import os
import shutil
import hashlib
import datetime
import ZipHelper
import xmlParamsHealper
import JsonParamsHelper

class PublisherHealperConfig:
    """Class for setting up publisher Healper"""
    moduleName = "" # name of the module to be processing
    destinationDir = "" # folder to deploy to
    sourceDir = "" # folder from with to deploy
    doBackup = False # does publisher need to make a backup
    bacupType = 'Folder' # "Folder" to backup whole destinationDir, "Files" to backup only owerwritten files
    backupFolder = "" # folder in witch the backup will be stored
    includeFolders = [] # folders to publish
    # includeFolders = [ # SAMPLE
    #     {
    #         "folder": "testFolder", # Folder to include. Do not provide this for source root folder
    #         "recursive": True, # Process folder recursively? Default is False
    #         "includeExtensions": ["py"], # extensions to be included. Do not provide this for all files in Folder
    #         "excludeExtensions": ["pyc"], # extensions to be excluded. Do not provide this if dont needed
    #         "clearExtensions": ["pyc"], # extensions to be deleted from destination. Do not provide this if dont needed
    #         "includeFiles": ["somespecificfile.py"], # files to be specificly included. Do not provide this if dont needed
    #         "excludeFiles": ["somespecificfile.py"], # files to be specificly excluded. Do not provide this if dont needed
    #         "clearFiles": ["somespecificfile.py"], # files to be specificly deleted from destination. Do not provide this if dont needed
    #         "renameFiles": {"somefilenamefromtorename.py": "somewithdifferentname.py"}
    #     }
    # ]
    configFilesJson = [] # config files of type Json to be processed

    configFilesXML = [] # config files of type XML to be processed
    # configFilesXML = [ # SAMPLE
    #     {
    #         "file": "Web.config", # relative path in destination
    #         "changes": [ # List of changes to be made
    #             {
    #                 "xpath": '/Test/Level1/Level2/Level3', # xpath to tag to be changed (first found will be processed)
    #                 "atribute": "someatribute", # Atribute to be updated. Do not provide this if tag text is to be updated
    #                 "value": "value to be writen" # value to be writen
    #             }
    #         ]
    #     }
    # ]
    replacementMap = {}
    # replacementMap = { # SAMPLE
    #     'test.json': {
    #         '[find sting to replace]': 'replacement value'
    #     }
    # }
class PublisherHealper(object):
    """Class for easing the Rar file operations"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """
        self.backup_zip_file = ''

    def Deply(self, config):
        """Does the dployment

        Args:
            self: The reserved object 'self'
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        print u'... start publish for {}'.format(config.moduleName)
        self.backup_zip_file = "{}_{}.zip".format(config.moduleName, _now_for_file())

        destination_dir = config.destinationDir
        if not os.path.exists(destination_dir):
            raise AttributeError(u'destination folder {} not found'.format(destination_dir))

        self.__create_backup(config)

        for folder in config.includeFolders:
            self.__do_deploy(folder, config)

        self.__do_process_xml(config)
        self.__do_process_json(config)
        self.__do_string_repalce(config)

    def __create_backup(self, config):
        """Does the backup creation

        Args:
            self: The reserved object 'self'
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        if hasattr(config, "doBackup"):
            if config.bacupType.upper() == 'FOLDER':
                backup_dir = config.backupFolder
                if not os.path.exists(backup_dir):
                    os.makedirs(backup_dir)
                    print u'... created backup folder {}'.format(backup_dir)

                backup_file_name = os.path.join(backup_dir, self.backup_zip_file)
                ZipHelper.ZipHelper().CompressDir(config.destinationDir, backup_file_name)
                print u'... backup created!'


    def __create_backup_one_file(self, file_path, config):
        """Does the backup creation for one file

        Args:
            self: The reserved object 'self'
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        if hasattr(config, "doBackup"):
            if config.bacupType.upper() == 'FILES':
                backup_dir = config.backupFolder
                if not os.path.exists(backup_dir):
                    os.makedirs(backup_dir)
                    print u'... created backup folder {}'.format(backup_dir)

                backup_file_name = os.path.join(backup_dir, self.backup_zip_file)
                ZipHelper.ZipHelper().CompressFileList(
                    filePathList=[file_path],
                    zipFileName=backup_file_name,
                    base_dir=config.destinationDir,
                    append=os.path.exists(backup_file_name))
                print u'... file {} backup created!'.format(file_path)

    def __do_deploy(self, folder, config):
        """Does the backup creation

        Args:
            self: The reserved object 'self'
            folder ([string]): relative path to folder to be processed
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        self.__clear(folder, config)
        files_to_copy = self.__files_to_copy(folder, config)
        self.__do_copy_files_to_dest(folder, files_to_copy, config)

    def __clear(self, folder, config):
        """Clears unnececery files

        Args:
            self: The reserved object 'self'
            folder ([string]): relative path to folder to be processed
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        if folder.has_key("clearExtensions") or folder.has_key("clearFiles"):
            clear_extensions = folder[u'clearExtensions'] if folder.has_key("clearExtensions") else []
            clear_files = folder[u'clearFiles'] if folder.has_key("clearFiles") else []
            recursive = folder[u'recursive'] if folder.has_key("recursive") else False

            source_dir = os.path.join(config.sourceDir, folder["folder"]) if folder.has_key("folder") else config.sourceDir
            destination_dir = os.path.join(config.destinationDir, folder["folder"]) if folder.has_key("folder") else config.destinationDir

            if not recursive:
                include_folders = []
            else:
                include_folders = _find_all_folders(destination_dir)
            include_folders.append(destination_dir)

            files_to_delete = []
            for infolder in include_folders:
                for ext in clear_extensions:
                    destination_folder = infolder.replace(source_dir, destination_dir)
                    if not os.path.exists(destination_folder):
                        os.makedirs(destination_folder)
                        print u'... output folder created {}'.format(destination_folder)
                    found_files = _find_file(os.path.join(destination_dir, infolder), ext)
                    if found_files:
                        files_to_delete = files_to_delete + found_files
                for file_to_clear in clear_files:
                    file_name = os.path.join(destination_dir, infolder, file_to_clear)
                    if os.path.exists(file_name):
                        files_to_delete.append(file_name)

            for file_to_delate in files_to_delete:
                os.remove(file_to_delate)
                print u'... file deleted      {}'.format(file_to_delate)

    def __files_to_copy(self, folder, config):
        """Finds files to be copyed

        Args:
            self: The reserved object 'self'
            folder ([string]): relative path to folder to be processed
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        recursive = folder[u'recursive'] if folder.has_key("recursive") else False

        source_dir = os.path.join(config.sourceDir, folder["folder"]) if folder.has_key("folder") else config.sourceDir
        destination_dir = os.path.join(config.destinationDir, folder["folder"]) if folder.has_key("folder") else config.destinationDir

        if not recursive:
            include_folders = []
        else:
            include_folders = _find_all_folders(source_dir)
        include_folders.append(source_dir)

        files_to_copy = []
        if folder.has_key("includeExtensions") or folder.has_key("includeFiles"):
            files_to_copy = self.__find_files_to_include(folder, include_folders, source_dir, destination_dir)
        else:
            files_to_copy = self.__find_all_files_to_include(folder, include_folders, source_dir, destination_dir)

        if folder.has_key("excludeExtensions") or folder.has_key("excludeFiles"):
            files_to_copy = self.__exclude_files(folder, files_to_copy)

        return files_to_copy

    def __find_files_to_include(self, folder, include_folders, source_dir, destination_dir):
        files_to_copy = []
        include_extensions = folder[u'includeExtensions'] if folder.has_key("includeExtensions") else []
        include_files = folder[u'includeFiles'] if folder.has_key("includeFiles") else []

        for infolder in include_folders:
            for ext in include_extensions:
                found_files = _find_file(infolder, ext)
                if found_files:
                    files_to_copy = files_to_copy + found_files
                    if not infolder == source_dir:
                        destination_folder = infolder.replace(source_dir, destination_dir)
                        if not os.path.exists(destination_folder):
                            os.makedirs(destination_folder)
                            print u'... output folder created {}'.format(destination_folder)
            for file_name in include_files:
                found_files = _find_file_by_name(infolder, file_name)
                if found_files:
                    files_to_copy = files_to_copy + found_files
                    if not infolder == source_dir:
                        destination_folder = infolder.replace(source_dir, destination_dir)
                        if not os.path.exists(destination_folder):
                            os.makedirs(destination_folder)
                            print u'... output folder created {}'.format(destination_folder)
        return files_to_copy

    def __find_all_files_to_include(self, folder, include_folders, source_dir, destination_dir):
        files_to_copy = []
        for infolder in include_folders:
            found_files = _find_all_files(os.path.join(source_dir, infolder))
            if found_files:
                files_to_copy = files_to_copy + found_files
                if not folder == source_dir:
                    destination_folder = os.path.join(destination_dir, os.path.basename(infolder))
                    if not os.path.exists(destination_folder):
                        os.makedirs(destination_folder)
                        print u'... output folder created {}'.format(destination_folder)
        return files_to_copy

    def __exclude_files(self, folder, files_to_copy):
        exclude_extensions = folder[u'excludeExtensions'] if folder.has_key("excludeExtensions") else []
        exclude_files = folder[u'excludeFiles'] if folder.has_key("excludeFiles") else []

        for ext in exclude_extensions:
            files_to_copy = list(fn for fn in files_to_copy if not os.path.basename(fn).lower().endswith('.' + (ext.lower())))

        for exclude_file in exclude_files:
            files_to_copy = list(fn for fn in files_to_copy if not os.path.basename(fn).lower() == exclude_file.lower())
        return files_to_copy

    def __do_copy_files_to_dest(self, folder, files_to_copy, config,):
        """Finds files to be copyed

        Args:
            self: The reserved object 'self'
            folder ([string]): relative path to folder to be processed
            files_to_copy ([list]): path of files to be copyed
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        source_dir = os.path.join(config.sourceDir, folder["folder"]) if folder.has_key("folder") else config.sourceDir
        destination_dir = os.path.join(config.destinationDir, folder["folder"]) if folder.has_key("folder") else config.destinationDir

        for copy_file in files_to_copy:
            dest_file = copy_file
            dest_file = dest_file.replace(source_dir, destination_dir)
            dest_file = self.__rename_file_if_needed(dest_file, folder)
            replaced = False
            thesame = False

            if os.path.exists(dest_file):
                copy_hash = _md5(copy_file)
                dest_hash = _md5(dest_file)
                if copy_hash <> dest_hash:
                    self.__create_backup_one_file(dest_file, config)
                    os.remove(dest_file)
                    replaced = True
                else:
                    thesame = True
            if not thesame:
                shutil.copy2(copy_file, dest_file)
                if not replaced:
                    print u'... file copy    {}'.format(dest_file)
                else:
                    print u'... file replace {}'.format(dest_file)

    def __rename_file_if_needed(self, dest_file, folder):
        rename_files = folder[u'renameFiles'] if folder.has_key("renameFiles") else {}
        dir_name, file_name = os.path.split(dest_file)
        for rename_file in rename_files:
            if file_name.upper() == rename_file.upper():
                return os.path.join(dir_name, rename_files[rename_file])
        return dest_file

    def __do_process_xml(self, config):
        """Changes required values in config xml

        Args:
            self: The reserved object 'self'
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        for config_file in config.configFilesXML:
            params_helper = xmlParamsHealper.XMLParams(None, None, os.path.join(config.destinationDir, config_file['file']))
            params_helper.GetParams()
            for change in config_file['changes']:
                is_string = False
                do_append = False
                if change.has_key('string'):
                    if change['string']:
                        is_string = True
                if change.has_key('append'):
                    if change['append']:
                        do_append = True
                if do_append:
                    attribute = None
                    key = None
                    if change.has_key("atribute"):
                        attribute = change['atribute']
                    if change.has_key("appendKey"):
                        key = change['appendKey']
                    params_helper.AppendValueByPath(change['xpath'], key, change['value'], attribute, isString=is_string)
                else:
                    if change.has_key("atribute"):
                        params_helper.UpdateAtributeByPath(change['xpath'], change['atribute'], change['value'])
                    else:
                        params_helper.UpdateValueByPath(change['xpath'], change['value'])
            params_helper.WriteParams()
            print u'... config file {} updated'.format(config_file['file'])

    def __do_process_json(self, config):
        """Changes required values in config xml

        Args:
            self: The reserved object 'self'
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        for config_file in config.configFilesJson:
            params_helper = JsonParamsHelper.JsonParams(None, None, os.path.join(config.destinationDir, config_file['file']))
            params_helper.GetParams()
            for change in config_file['changes']:
                is_json = False
                do_append = False
                if change.has_key('json'):
                    if change['json']:
                        is_json = True
                if change.has_key('append'):
                    if change['append']:
                        do_append = True
                if do_append:
                    params_helper.AppendValueByPath(change['xpath'], change['appendKey'], change['value'], is_json)
                else:
                    params_helper.UpdateValueByPath(change['xpath'], change['value'], is_json)
            params_helper.WriteParams(False)
            print u'... config file {} updated'.format(config_file['file'])

    def __do_string_repalce(self, config):
        """Replace required values by sring replacement

        Args:
            self: The reserved object 'self'
            config ([PublisherHealperConfig]): Configuration of deplyment
        """
        for file_name in config.replacementMap:
            replacement_map = config.replacementMap[file_name]
            path = os.path.join(config.destinationDir, file_name)
            _replace_in_file(path, replacement_map)
            print u'... file {} replaced strings'.format(path)

def _replace_in_file(path, replace_map):
    """replaces values in files using replace_map
    """
    with codecs.open(path, 'r') as f:
        newlines = []
        for line in f.readlines():
            for key, value in replace_map.items():
                line = line.replace(key, value)
            newlines.append(line)
    with open(path, 'w') as f:
        for line in newlines:
            f.write(line)

def _find_all_files(directory):
    """Finds files in the directory

    Args:
        dir: The directory in which to look for the file
    """
    found_files = [directory + "\\" + fn
                   for fn in os.listdir(directory)]
    found_files.sort()
    return found_files

def _find_file(directory, ext):
    """Finds files in the directory

    Args:
        Dir: The directory in which to look for the file
        Ext: The extension to search for
    """
    found_files = [directory + "\\" + fn
                   for fn in os.listdir(directory) if fn.lower().endswith('.' + (ext.lower()))]
    found_files.sort()
    return found_files

def _find_file_by_name(directory, file_name):
    """Finds files in the directory

    Args:
        Dir: The directory in which to look for the file
        fileName: File name to search for
    """
    found_files = [directory + "\\" + fn
                   for fn in os.listdir(directory) if fn.lower() == file_name.lower()]
    found_files.sort()
    return found_files

def _find_all_folders(directory):
    """Finds files in the directory

    Args:
        Dir: The directory in which to look for the file
        Ext: The extension to search for
    """
    return [x[0] for x in os.walk(directory)]

def _md5(filename):
    """calculates file md5 cheksumm

    Args:
        fname ([string]): File path

    Returns:
        [string]: hex digest
    """
    hash_md5 = hashlib.md5()
    with open(filename, "rb") as opened_file:
        for chunk in iter(lambda: opened_file.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()

def _now_for_file():
    """returns date now formated for filename

    Returns:
        [string]: [date reprezentation as string]
    """
    return datetime.datetime.strftime(datetime.datetime.now(), "%Y%m%d_%H%M%S")



# -*- coding: utf-8 -*-
"""
     Module for operations with ArcGIS Server services
"""

import urllib
import json
import urllib2
import ssl
import urlparse
from collections import OrderedDict
from ntlm import HTTPNtlmAuthHandler
import MyError

class AGServerHelperNTLM(object):
    """Class for operations with ArcGIS Server services"""

    def __init__(self,
                 username,
                 password,
                 ags_admin_url,
                 tool=None,
                 basic=False,
                 allowunverifiedssl=False):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            username: ArcGIS Server administrator username
            password: ArcGIS Server administrator password
            ags_admin_url: ArcGIS server rest admin url
            Tool: GISPython tool (optional)
            basic: bool indicating that Basic autentification will be used instead of NTLM
        """
        self.username = username
        self.password = password
        self.ags_admin_url = ags_admin_url
        self.serverurl = ags_admin_url
        if self.ags_admin_url.endswith("/"):
            self.ags_admin_url = self.ags_admin_url[:-1]
        self.Tool = tool

        self.ags_admin_url = urlparse.urljoin(self.ags_admin_url, '/arcgis/admin')

        if allowunverifiedssl:
            try:
                _create_unverified_https_context = ssl._create_unverified_context
            except AttributeError:
                # Legacy Python that doesn't verify HTTPS certificates by default
                pass
            else:
                # Handle target environment that doesn't support HTTPS verification
                ssl._create_default_https_context = _create_unverified_https_context

        passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passman.add_password(None, self.ags_admin_url, self.username, self.password)

        if not basic:
            auth_handler = HTTPNtlmAuthHandler.HTTPNtlmAuthHandler(passman)
        else:
            auth_handler = urllib2.HTTPBasicAuthHandler(passman)

        opener = urllib2.build_opener(auth_handler)
        urllib2.install_opener(opener)


    def __request_from_server(self, adress, params, content_type='application/json', method="POST"):
        """Function for ntlm request creation

        Args:
            self: The reserved object 'self'
            adress: Adress of request
            params: Params as dictionary
            content_type: Http content type
            method: Http method

        Returns:
            Response string
        """
        data = urllib.urlencode(params)
        url_address = urlparse.urljoin(self.ags_admin_url + "/", adress)
        req = urllib2.Request(url=url_address, data=data)
        req.add_header('Content-Type', content_type)
        req.get_method = lambda: method
        response = urllib2.urlopen(req)
        response_string = response.read()
        if not response.code == 200:
            raise MyError.MyError("Error: in getting url: {0}?{1} {2} message: {3}".format(
                url_address, data, method, response.msg))
        return response_string

    def __assert_json_success(self, data):
        """Function for aserting json request state

        Args:
            self: The reserved object 'self'
            data: Request response string

        Returns:
            boolean False if request has errors
        """
        obj = json.loads(data)
        if 'status' in obj and obj['status'] == "error":
            raise MyError.MyError("Error: JSON object returns an error. " + str(obj))
        else:
            return True

    def __process_folder_string(self, folder):
        """Function for processing folder name string

        Args:
            self: The reserved object 'self'
            folder: folder string

        Returns:
            corrected folder string
        """
        if folder is None:
            folder = "ROOT"
        if folder.upper() == "ROOT":
            folder = ""
        else:
            folder += "/"
        return folder

    def startService(self, folder, service):
        """Starts AGS Service

        Args:
            folder (string): AGS folder of the service. (CASE sensitive) Use ROOT for services without folder.
            service (string): Service name (CASE sensitive)
        """
        self.__start_stop_service(folder, service, "start")

    def stopService(self, folder, service):
        """Stops AGS Service

        Args:
            folder (string): AGS folder of the service. (CASE sensitive) Use ROOT for services without folder.
            service (string): Service name (CASE sensitive)
        """
        self.__start_stop_service(folder, service, "stop")

    def __start_stop_service(self, folder, service, action):
        folder = self.__process_folder_string(folder)
        folder_url = "services/" + folder + service + "/" + action + '?f=json'
        params = {}
        data = self.__request_from_server(folder_url, params)

        if not self.__assert_json_success(data):
            raise MyError.MyError("Error when reading folder information. " + str(data))
        else:
            if self.Tool != None:
                self.Tool.AddMessage("Service {}{} {}  done successfully ...".format(
                    folder, service, action))

    def getServiceList(self, folder):
        """Retrieve ArcGIS server services

        Args:
            self: The reserved object 'self'
            folder: Folder of the service (ROOT for root services)
        """
        services = []
        folder = self.__process_folder_string(folder)
        url = "services/{}?f=json".format(folder)
        params = {}
        data = self.__request_from_server(url, params)

        try:
            service_list = json.loads(data)
        except urllib2.URLError, exception:
            raise MyError.MyError(exception)

        for single in service_list["services"]:
            services.append(folder + single['serviceName'] + '.' + single['type'])

        folder_list = service_list["folders"] if u'folders' in service_list != False else []
        if u'Utilities' in service_list != False:
            folder_list.remove("Utilities")
        if u'System' in service_list != False:
            folder_list.remove("System")

        if folder_list:
            for subfolder in folder_list:
                url = "services/{}?f=json".format(subfolder)
                data = self.__request_from_server(url, params)
                subfolder_list = json.loads(data)

                for single in subfolder_list["services"]:
                    services.append(subfolder + "//" + single['serviceName'] + '.' + single['type'])

        if not services:
            if self.Tool != None:
                self.Tool.AddMessage("No services found")
        else:
            if self.Tool != None:
                self.Tool.AddMessage("Services on " + self.serverurl +":")
            for service in services:
                status_url = "services/{}/status?f=json".format(service)
                data = self.__request_from_server(status_url, params)
                status = json.loads(data)
                if self.Tool != None:
                    self.Tool.AddMessage("  " + status["realTimeState"] + " > " + service)

        return services

    def GetServerJson(self, server_service):
        """Retrieve service parameters

        Args:
            self: The reserved object 'self'
            serverService: Service which parameter configuration shall be retrieved

        Returns:
            json data object
        """
        service_url = "services/" + server_service + "?f=json"
        params = {}
        data = self.__request_from_server(service_url, params, method='GET')

        if not self.__assert_json_success(data):
            raise MyError.MyError(
                u'...Couldn\'t retrieve service parameter configuration: {}\n'.format(data))
        else:
            if self.Tool != None:
                self.Tool.AddMessage(u'...Service parameter configuration successfully retrieved\n')

        data_object = json.loads(data)
        return data_object

    def publishServerJson(self, service, data_object):
        """Publish service parameters to server

        Args:
            self: The reserved object 'self'
            service: Service which parameter configuration shall be renewed
            data_object: Parameter configuration
        """

        updated_svc_json = json.dumps(data_object)
        edit_svc_url = "services/" + service + "/edit"
        params = {'f': 'json', 'service': updated_svc_json}
        edit_data = self.__request_from_server(edit_svc_url,
                                               params,
                                               'application/x-www-form-urlencoded',
                                               method='POST')

        if not self.__assert_json_success(edit_data):
            if self.Tool != None:
                self.Tool.AddMessage(
                    u'...Service configuration renewal error: {}\n'.format(edit_data))
        else:
            if self.Tool != None:
                self.Tool.AddMessage(u'...Service configuration succesfully renewed\n')

        return

    def getServiceFromServer(self, services, service, serviceDir):
        """Retrieve the full service name from the server

        Args:
            self: The reserved object 'self'
            services: List of all services on server
            service: Name of the service from which to get corresponding name from the server services list
            serviceDir: Name of the service directory which is shown in the configuration of services to be published on the server
        """
        server_service = ''
        if serviceDir is None:
            config_service = service
        else:
            config_service = serviceDir + "//" + service
        for server_service in services:
            if server_service.split('.')[0].upper() == config_service.upper():
                return server_service

        return ''

    def getServicePermisions(self, folder, service):
        """Check service permisions

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service

        Returns: Dictionary of service principals
        """
        folder = self.__process_folder_string(folder)
        status_url = "services/" + folder + service + "/permissions?f=pjson"
        params = {}
        permisions_data = self.__request_from_server(status_url, params, method='GET')

        if not self.__assert_json_success(permisions_data):
            raise MyError.MyError(
                "Error while retrieving permisions information for {}.".format(service))

        statusdata_object = json.loads(permisions_data)
        return_dict = OrderedDict()
        for permision in statusdata_object["permissions"]:
            principal = permision["principal"]
            if permision.has_key("permission"):
                if permision["permission"].has_key("isAllowed"):
                    return_dict[principal] = permision["permission"]["isAllowed"]
                else:
                    return_dict[principal] = True
        return return_dict

    def addServicePermisions(self, folder, service, principal, is_allowed='true'):
        """Add service permisions

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service
            principal: The name of the role for whom the permission is being assigned.
            is_allowed: Tells if access to a resource is allowed or denied.
        """
        urlparams = urllib.urlencode({'principal':principal, 'isAllowed':is_allowed, 'f':'json'})
        folder = self.__process_folder_string(folder)
        status_url = "services/" + folder + service + "/permissions/add?{}".format(urlparams)
        params = {}
        permisions_data = self.__request_from_server(status_url, params, method='POST')

        if not self.__assert_json_success(permisions_data):
            raise MyError.MyError(
                "Error while setting permisions information for {} to {}.".format(
                    service, principal))

    def isServiceRunning(self, folder, service):
        """Check if service is running

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service

        Returns: True if is running
        """
        folder = self.__process_folder_string(folder)
        status_url = "services/" + folder + service + "/status?f=json"
        params = {}
        status_data = self.__request_from_server(status_url, params, method='GET')

        if not self.__assert_json_success(status_data):
            raise MyError.MyError(
                "Error while retrieving status information for {}.".format(service))

        statusdata_object = json.loads(status_data)
        if statusdata_object['realTimeState'] == "STOPPED":
            return False
        else:
            return True

    def GetDatasetNames(self, folder, service):
        """Retrieve the service Dataset Names from the server

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service

        Returns: list of strings
        """
        folder = self.__process_folder_string(folder)
        manifest_url = "services/" + folder + service + "/iteminfo/manifest/manifest.json?f=json"
        params = {}
        status_data = self.__request_from_server(manifest_url, params, method='GET')
        rezult = list()

        if not self.__assert_json_success(status_data):
            raise MyError.MyError(
                "Error while retrieving manifest information for {}.".format(service))

        statusdata_object = json.loads(status_data)
        for database in statusdata_object['databases']:
            rezult.append(database['onServerName'])

        return rezult

    def getRoles(self, pageSize=5000):
        """Retrieve the Role Names from the server

        Args:
            self: The reserved object 'self'

        Returns: list of strings
        """
        manifest_url = "security/roles/getRoles?startIndex=0&pageSize={}&f=json".format(pageSize)
        params = {}
        roles_data = self.__request_from_server(manifest_url, params, method='POST')
        rezult = list()

        if not self.__assert_json_success(roles_data):
            raise MyError.MyError("Error while retrieving role information.")

        roles = json.loads(roles_data)
        for role in roles['roles']:
            if role.has_key("description"):
                rezult.append({"rolename": role["rolename"],
                               "description": role["description"]})
            else:
                rezult.append({"rolename": role["rolename"]})


        return rezult

    def addRole(self, rolename, description=''):
        """Retrieve the Role Names from the server

        Args:
            self: The reserved object 'self'
            rolename: The name of the role. The name must be unique in the role store.
        """
        urlparams = urllib.urlencode({'rolename':rolename, 'description':description, 'f':'json'})
        manifest_url = "security/roles/add?{}".format(urlparams)
        params = {}
        roles_data = self.__request_from_server(manifest_url, params, method='POST')

        if not self.__assert_json_success(roles_data):
            raise MyError.MyError("Error while adding role {}".format(rolename))

    def removeRole(self, rolename):
        """Retrieve the Role Names from the server

        Args:
            self: The reserved object 'self'
            rolename: The name of the role.
        """
        urlparams = urllib.urlencode({'rolename':rolename, 'f':'json'})
        manifest_url = "security/roles/remove?{}".format(urlparams)
        params = {}
        roles_data = self.__request_from_server(manifest_url, params, method='POST')

        if not self.__assert_json_success(roles_data):
            raise MyError.MyError("Error while removing role {}".format(rolename))

    def getUsersWithinRole(self, rolename, maxCount=5000):
        """Retrieve the Role Names from the server

        Args:
            self: The reserved object 'self'

        Returns: list of strings
        """
        urlparams = urllib.urlencode({'rolename':rolename, 'maxCount':maxCount, 'f':'json'})
        manifest_url = "security/roles/getUsersWithinRole?{}".format(urlparams)
        params = {}
        users_data = self.__request_from_server(manifest_url, params, method='POST')
        rezult = list()

        if not self.__assert_json_success(users_data):
            raise MyError.MyError("Error while retrieving role user information.")

        users = json.loads(users_data)
        for user in users['users']:
            rezult.append(user)

        return rezult

    def addUsersToRole(self, rolename, users):
        """assign a role to multiple users with a single action

        Args:
            self: The reserved object 'self'
            rolename: The name of the role.
            users: A comma-separated list of user names. Each user name must exist in the user store.
        """
        urlparams = urllib.urlencode({'rolename':rolename, 'users':users, 'f':'json'})
        manifest_url = "security/roles/addUsersToRole?{}".format(urlparams)
        params = {}
        roles_data = self.__request_from_server(manifest_url, params, method='POST')

        if not self.__assert_json_success(roles_data):
            raise MyError.MyError(
                "Error while adding users [{1}] to a role {0}".format(rolename, users))

    def removeUsersFromRole(self, rolename, users):
        """Removes a role assignment from multiple users

        Args:
            self: The reserved object 'self'
            rolename: The name of the role.
            users: A comma-separated list of user names. Each user name must exist in the user store.
        """
        urlparams = urllib.urlencode({'rolename':rolename, 'users':users, 'f':'json'})
        manifest_url = "security/roles/removeUsersFromRole?{}".format(urlparams)
        params = {}
        roles_data = self.__request_from_server(manifest_url, params, method='POST')

        if not self.__assert_json_success(roles_data):
            raise MyError.MyError(
                "Error while removing users [{1}] from a role {0}".format(rolename, users))

    def GetDatasetNamesWithObjects(self, folder, service):
        """Retrieve the service Dataset Names from the server

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service

        Returns: list of strings
        """
        folder = self.__process_folder_string(folder)
        manifest_url = "services/" + folder + service + "/iteminfo/manifest/manifest.json?f=json"
        params = {}
        status_data = self.__request_from_server(manifest_url, params, method='GET')
        rezult = list()

        if not self.__assert_json_success(status_data):
            raise MyError.MyError("Error while retrieving manifest information for " + service + ".")

        statusdata_object = json.loads(status_data)
        for database in statusdata_object['databases']:
            dataset_names = [d['onServerName'] for d in database['datasets']]
            item = {"database": database['onServerName'], "datasets":dataset_names}
            rezult.append(item)

        return rezult

    def GetRightsGroupsNames(self, folder, service):
        """Retrieve the service permission role names from service

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service

        Returns: list of strings
        """
        folder = self.__process_folder_string(folder)
        manifest_url = "services/" + folder + service + "/permissions?f=json"
        params = {}
        status_data = self.__request_from_server(manifest_url, params, method='GET')
        rezult = list()

        if not self.__assert_json_success(status_data):
            raise MyError.MyError(
                "Error while retrieving permissions information for {}.".format(service))

        permissions = json.loads(status_data)
        for permission in permissions['permissions']:
            rezult.append(permission['principal'])

        return rezult

    def GetServiceInfo(self, folder):
        """Retrieve the Folder List from the server

        Args:
            self: The reserved object 'self'
            folder: Service directory

        Returns: list of service objects
        """
        folder = self.__process_folder_string(folder)
        manifest_url = "services/" + folder + "/?f=json"
        params = {}
        status_data = self.__request_from_server(manifest_url, params, method='GET')
        rezult = list()

        if not self.__assert_json_success(status_data):
            raise MyError.MyError("Error while retrieving folder information.")

        statusdata_object = json.loads(status_data)
        rezult.append(statusdata_object)
        folderlist = list()
        for folder_detail in statusdata_object['foldersDetail']:
            folderlist.append(folder_detail['folderName'])

        for subfolder in folderlist:
            if not (subfolder.upper() == 'System'.upper()
                    or subfolder.upper() == 'Utilities'.upper()):
                manifest_url = "services/" + subfolder + "/?f=json"
                status_data = self.__request_from_server(manifest_url, params, method='GET')
                statusdata_object = json.loads(status_data)
                rezult.append(statusdata_object)

        return rezult



# -*- coding: utf-8 -*-
"""
     Module for e-mail operations
"""
import os

class MailHelper:
    """Class for easing the SMTP operations"""

    def __init__(self, From, recipients, Subject, Text):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            From: E-mail adress from which to send
            recipients: Recipient array
            Subject: E-mail subject
            Text: E-mail text
        """
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        self.msg = MIMEMultipart()
        self.msg['Subject'] = Subject
        self.msg['To'] = ", ".join(recipients)
        self.msg.attach(MIMEText(Text, 'plain', 'utf-8'))
        self.From = From
        self.recipients = recipients

    def AttachFile(self, FilePath):
        """Procedure to add attachments

        Args:
            self: The reserved object 'self'
            FilePath: Path to the attachment
        """
        from email.mime.base import MIMEBase
        from email import encoders
        part = MIMEBase('application', "octet-stream")
        with open(FilePath, "rb") as _file:
            part.set_payload(_file.read())
        encoders.encode_base64(part)
        part.add_header('Content-Disposition', 'attachment; filename="{0}"'.format(os.path.basename(FilePath)))
        self.msg.attach(part)

    def SendMessage(self, Mailserver, port=None, user=None, password=None, useTLS=False, useSSL=False):
        """Procedure for sending an e-mail

        Args:
            self: The reserved object 'self'
            Mailserver: Mailserver name
            port: Mailserver port number
            user: Username
            password: Password
            useTLS: Use TLS (Default = False)
            useSSL: Use SSL (Default = False)
        """
        import smtplib
        if useSSL:
            mailer = smtplib.SMTP_SSL()
        else:
            mailer = smtplib.SMTP()
        if port != None:
            mailer.connect(Mailserver, port)
        else:
            mailer.connect(Mailserver)
        if useTLS:
            mailer.starttls()
        if user != None:
            mailer.login(user, password)
        mailer.sendmail(self.From, self.recipients, self.msg.as_string())
        mailer.close()

class GISPythonMailHelper(MailHelper):
    """MailHelper wrapper class, which acquires parameters from the GISPython parameter file"""

    def __init__(self, Pr, recipients, Subject, Text):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            Pr: Parameter file with corresponding parameters
            recipients: Recipient array
            Subject: E-mail subject
            Text: E-mail text
        """
        mailer = MailHelper(Pr.MailFromAdress, recipients, Subject, Text)

        # Check if attributes exists
        if hasattr(Pr, 'MailserverPort'):
            port = Pr.MailserverPort
        else:
            port = None

        if hasattr(Pr, 'MailserverUseTLS'):
            useTLS = Pr.MailserverUseTLS
        else:
            useTLS = None

        if hasattr(Pr, 'MailserverUseSSL'):
            useSSL = Pr.MailserverUseSSL
        else:
            useSSL = None

        if hasattr(Pr, 'MailserverUser'):
            user = Pr.MailserverUser
            pwd = Pr.MailserverPWD
        else:
            user = None
            pwd = None

        # Send a message
        mailer.SendMessage(Pr.Mailserver, port, user, pwd, useTLS, useSSL)



# -*- coding: utf-8 -*-
"""
    Test procedure module
"""

import unittest
import GISPythonModule

class Pr(object):
    """Defines test parameters"""
    OutDir = 'C:\\GIS\\Log\\Outlog\\' # Output file directory
    OutDirArh = 'C:\\GIS\\Log\\Outlog\\Archive\\' # Output file archive directory
    ErrorLogDir = 'C:\\GIS\\Log\\ErrorLog\\' # Error output file directory
    ErrorLogDirArh = 'C:\\GIS\\Log\\ErrorLog\\Archive\\' # Error output file archive directory
    encodingPrimary = 'cp1257' # Windows charset
    encodingSecondary = 'cp775' # Shell charset

class GISTools_unittest(unittest.TestCase):
    """GEOPython unit test class"""

    def setUp(self):
        """The Test setting up procedure"""
        print u"Preparing for tests..."
        self.Module = GISPythonModule.GISPythonModule('Unittest', Pr)
        self.Module.initModule()
        #don't catch errors in the file
        return super(GISTools_unittest, self).setUp()

    def tearDown(self):
        """"The Test tear down - cleaning up objects after test"""
        print u"Cleaning up after tests..."
        self.Module.MyDispose()
        return super(GISTools_unittest, self).tearDown()

    def test_Tool_init(self):
        """Check if it is possible to get the geoprocessor object"""
        print u"Check if it is possible to get the geoprocessor object ... OK"

    def test_shellRun(self):
        """Check the shell execution commands"""
        self.Module.Tool.runShell("Dir")
        print u"Check if it is possible to get the geoprocessor object ... OK"

if __name__ == "__main__":
    print ""
    unittest.main()
    raw_input("Press Enter to continue...")



# -*- coding: utf-8 -*-
"""
     Data synchronization module 2 (synchronizes data between tables)

     Second version is ment for advanced scenarious, but opereates data inMemory so it,s not intended for large data sets.
"""
import TimerHelper

class GDPSyncroniserHelper2(object):
    """ESRI table synchronization class 2

       Second version is ment for advanced scenarious, but opereates data inMemory so it,s not intended for large data sets
    """

    def __init__(self, gp, Tool=None):
        """Class initialization operation

        Args:
            self: The reserved object 'self'
            gp: ArcPy GP object
        """
        self.gp = gp
        if Tool != None:
            self.isTool = True
        else:
            self.isTool = False
        self.Tool = Tool

    def AddWarning(self, str):
        """Wrapper for the 'AddWarning' procedure

        Args:
            self: The reserved object 'self'
            str: Output string
        """
        if self.isTool:
            self.Tool.AddWarning(str)
        else:
            self.gp.AddWarning(str)

    def AddMessage(self, str):
        """Wrapper for the 'AddMessage' procedure

        Args:
            self: The reserved object 'self'
            str: Output string
        """
        if self.isTool:
            self.Tool.AddMessage(str)
        else:
            self.gp.AddMessage(str)

    def DoSync(self, definition, workspace=None, startEditing=False, startOperation=False, with_undo=False, multiuser=False):
        """Data synchronization procedure

        Args:
            self: The reserved object 'self'
            definition: 'SyncDefinition' object which describes the synchronization parameters
            workspace: DB in which to start data editing
            startEditing: Start the edit session in the DB specified in 'workspace' parameter? (Default = False)
            startOperation: Start the edit operation in the DB specified in 'workspace' parameter? (Default = False)
            with_undo: Sets whether the undo and redo stacks are enabled or disabled for an edit session. (Default = False)
            multiuser: Sets whether a DB contains a nonversioned, or versioned dataset. (Default = False)

        Returns:
            * output - list of dictionary items containig:
                - id: record id, 
                - syncResult: notInitialized,  synchronized, inserted, error
                - error: error mesage or '',
                - updated: true if inserted or updated
        """

        # move parameters to self
        self.definition = definition
        self.workspace = workspace
        self.startEditing = startEditing
        self.startOperation = startOperation
        self.with_undo = with_undo
        self.multiuser = multiuser

        inputAsTableObject = False

        if self.definition.SourceToDestination():
            if not self.definition.inTableObject == list():
                inputAsTableObject = True

        # set up fields parameters
        if not inputAsTableObject:
            self.definition.inTableFields = (self.definition.inTableJoinField,) + self.definition.inTableFields
        self.definition.outTableFields = (self.definition.outTableJoinField,) + self.definition.outTableFields

        # Ouput Sync process description
        self.AddMessage(u'>>>>Start the synchronization of the tables {0} and {1}'.format(self.definition.inTable, self.definition.outTable))
        self.AddMessage(u'>>>>Synchronization mode is "{0}"'.format(self.definition.mode))
        if inputAsTableObject:
            self.AddMessage(u'>>>>For input Will be used Python data list')
        else:
            self.AddMessage(u'>>>>>>>>Join {0}.{1}={2}.{3}'.format(self.definition.inTable, self.definition.inTableJoinField, self.definition.outTable, self.definition.outTableJoinField))
            self.AddMessage(u'>>>>>>>>Input fields: [{0}]'.format(", ".join(self.definition.inTableFields)))
            if self.definition.HasInTableQuery():
                self.AddMessage(u'>>>>>>>>Input query: [{0}]'.format(self.definition.inTableQuery))
        self.AddMessage(u'>>>>>>>>Output fields: [{0}]'.format(", ".join(self.definition.outTableFields)))
        if self.definition.HasOutTableQuery():
            self.AddMessage(u'>>>>>>>>Output query: [{0}]'.format(self.definition.outTableQuery))
        if self.definition.SourceToDestination():
            self.AddMessage(u'>>>>>>>>Do create new records: [{0}]'.format(self.definition.createNew))
        if self.definition.SourceToDestination():
            self.AddMessage(u'>>>>>>>>Do allow one to many join records: [{0}]'.format(self.definition.allowMultiple))

        # organize editor start
        if startEditing or startOperation:
            self.edit = self.gp.da.Editor(workspace)
        if startEditing:
            self.edit.startEditing(with_undo, multiuser)
        if startOperation:
            self.edit.startOperation()

        output = None

        # Sync process
        if self.definition.SourceToDestination():
            if not inputAsTableObject:
                output = self.__SyncSourceToDestination()
            else:
                output = self.__SyncSourceToDestination_TableObject()
        else:
            output = self.__SyncDestinationToSource()

        # organize editor stop
        if startOperation:
            self.edit.stopOperation()
        if startEditing:
            self.edit.stopEditing(True)

        # end
        return output

    def __SyncSourceToDestination_TableObject(self):
        """Procedure performs syncronization of type Source To Destination

        Args:
            self: The reserved object 'self'
       
        Returns:
            * output - list of dictionary items containig:
                - id: record id, 
                - syncResult: notInitialized,  synchronized, inserted, error
                - error: error mesage or '',
                - updated: true if inserted or updated
        """
        # get summary record count
        recordCount = 0
        recordCount = len(self.definition.inTableObject)
        self.AddMessage(u'>>>> ...Records to process {0}'.format(recordCount))

        rezultList =[]

        updatedCount = 0
        processedCount = 0
        processedCountOperation = 0
        errCount = 0

        T = TimerHelper.TimerHelper()
        for inRow in self.definition.inTableObject:
            syncItem = SyncItem2()
            syncItem.inSyncRow = inRow
            syncItem.id = inRow[0]

            with self.gp.da.UpdateCursor(self.definition.outTable, self.definition.outTableFields, u'{0} = {2}{1}{2}'.format(self.definition.outTableJoinField, syncItem.id, self.definition.idvalueseparator)) as outCur:
                recordsFoundCount = 0
                for outRow in outCur:
                    #print "rec {0} - {1}".format(u'{0} = {2}{1}{2}'.format(self.definition.outTableJoinField, syncItem.id, self.definition.idvalueseparator), (recordsFoundCount == 0 or self.definition.allowMultiple)) 
                    if recordsFoundCount == 0 or self.definition.allowMultiple:
                        recordsFoundCount = recordsFoundCount + 1
                        if recordsFoundCount>1:
                            syncItem = SyncItem2()
                            syncItem.inSyncRow = inRow
                            syncItem.id = inRow[0]
                        syncItem.outSyncRow = outRow
                        syncItem.DoSyncFields()
                        if syncItem.updated:
                            # self.AddMessage('...Updated')
                            outCur.updateRow(syncItem.outSyncRow)
                            updatedCount = updatedCount + 1
                            syncItem.ClerRowInfo()
                            syncItem.syncResult = 'synchronized'
                            rezultList.append(syncItem.GetRowStatussInfo())
                    else:
                        syncItem = SyncItem2()
                        syncItem.inSyncRow = inRow
                        syncItem.id = inRow[0]
                        errCount = errCount + 1
                        syncItem.error =  u"Error: row in the results table is not unique " + self.definition.messageDefinition.format(*syncItem.inSyncRow)
                        self.AddMessage(u'        ...{0}'.format(syncItem.error))
                        syncItem.ClerRowInfo()
                        syncItem.syncResult = 'error'
                        rezultList.append(syncItem.GetRowStatussInfo())

            if recordsFoundCount == 0 and self.definition.createNew:
                    with self.gp.da.InsertCursor(self.definition.outTable, self.definition.outTableFields) as insCur:
                        newRow = [None for _ in range(len(syncItem.inSyncRow))]
                        syncItem.outSyncRow = newRow
                        syncItem.DoSyncFields()
                        if syncItem.updated:
                            insCur.insertRow(syncItem.outSyncRow)
                            updatedCount = updatedCount + 1
                            syncItem.ClerRowInfo()
                            syncItem.syncResult = 'inserted'
                            rezultList.append(syncItem.GetRowStatussInfo())
            if recordsFoundCount == 0 and not self.definition.createNew:
                errCount = errCount + 1
                syncItem.error =  u"Error: no related records found in the results table " + self.definition.messageDefinition.format(*syncItem.inSyncRow)
                self.AddMessage(u'        ...{0}'.format(syncItem.error))
                syncItem.id = inRow[0]
                syncItem.ClerRowInfo()
                syncItem.syncResult = 'error'
                rezultList.append(syncItem.GetRowStatussInfo())

            processedCount = processedCount + 1
            processedCountOperation = processedCountOperation + 1

            if processedCountOperation >= 1000:
                if self.isTool:
                    self.AddMessage(u'>>>> ...Processed [{0}] from [{1}], stored [{2}], faulty [{3}] ({4})'.format(processedCount, recordCount, updatedCount, errCount, T.GetTimeReset()))
                processedCountOperation = 0
                if self.startOperation:
                    self.edit.stopOperation()
                if self.startOperation:
                    self.edit.startOperation()

        self.AddMessage(u'>>>> ...Processed [{0}], stored [{1}], faulty [{2}]'.format(processedCount, updatedCount, errCount))
        return rezultList

    def __SyncSourceToDestination(self):
        """Procedure performs syncronization of type Source To Destination

        Args:
            self: The reserved object 'self'
       
        Returns:
            * output - list of dictionary items containig:
                - id: record id, 
                - syncResult: notInitialized,  synchronized, inserted, error
                - error: error mesage or '',
                - updated: true if inserted or updated
        """
        # get summary record count
        recordCount = 0
        with self.gp.da.SearchCursor(self.definition.inTable, self.definition.inTableFields, self.definition.inTableQuery) as cur:
             for row in cur:
                recordCount = recordCount + 1
        self.AddMessage(u'>>>> ...Records to process {0}'.format(recordCount))

        rezultList =[]

        updatedCount = 0
        processedCount = 0
        processedCountOperation = 0
        errCount = 0

        with self.gp.da.SearchCursor(self.definition.inTable, self.definition.inTableFields, self.definition.inTableQuery) as inCur:
            T = TimerHelper.TimerHelper()
            for inRow in inCur:
                syncItem = SyncItem2()
                syncItem.inSyncRow = inRow
                syncItem.id = inRow[0]

                with self.gp.da.UpdateCursor(self.definition.outTable, self.definition.outTableFields, u'{0} = {2}{1}{2}'.format(self.definition.outTableJoinField, syncItem.id, self.definition.idvalueseparator)) as outCur:
                    recordsFoundCount = 0
                    for outRow in outCur:
                        #print "rec {0} - {1}".format(u'{0} = {2}{1}{2}'.format(self.definition.outTableJoinField, syncItem.id, self.definition.idvalueseparator), (recordsFoundCount == 0 or self.definition.allowMultiple)) 
                        if recordsFoundCount == 0 or self.definition.allowMultiple:
                            recordsFoundCount = recordsFoundCount + 1
                            if recordsFoundCount>1:
                                syncItem = SyncItem2()
                                syncItem.inSyncRow = inRow
                                syncItem.id = inRow[0]
                            syncItem.outSyncRow = outRow
                            syncItem.DoSyncFields()
                            if syncItem.updated:
                                outCur.updateRow(syncItem.outSyncRow)
                                updatedCount = updatedCount + 1
                                syncItem.ClerRowInfo()
                                syncItem.syncResult = 'synchronized'
                                rezultList.append(syncItem.GetRowStatussInfo())
                        else:
                            syncItem = SyncItem2()
                            syncItem.inSyncRow = inRow
                            syncItem.id = inRow[0]
                            errCount = errCount + 1
                            syncItem.error =  u"Error: row in the results table is not unique " + self.definition.messageDefinition.format(*syncItem.inSyncRow)
                            self.AddMessage(u'        ...{0}'.format(syncItem.error))
                            syncItem.ClerRowInfo()
                            syncItem.syncResult = 'error'
                            rezultList.append(syncItem.GetRowStatussInfo())

                if recordsFoundCount == 0 and self.definition.createNew:
                        with self.gp.da.InsertCursor(self.definition.outTable, self.definition.outTableFields) as insCur:
                            newRow = [None for _ in range(len(syncItem.inSyncRow))]
                            syncItem.outSyncRow = newRow
                            syncItem.DoSyncFields()
                            if syncItem.updated:
                                insCur.insertRow(syncItem.outSyncRow)
                                updatedCount = updatedCount + 1
                                syncItem.ClerRowInfo()
                                syncItem.syncResult = 'inserted'
                                rezultList.append(syncItem.GetRowStatussInfo())
                if recordsFoundCount == 0 and not self.definition.createNew:
                    errCount = errCount + 1
                    syncItem.error =  u"Error: no related records found in the results table " + self.definition.messageDefinition.format(*syncItem.inSyncRow)
                    self.AddMessage(u'        ...{0}'.format(syncItem.error))
                    syncItem.id = inRow[0]
                    syncItem.ClerRowInfo()
                    syncItem.syncResult = 'error'
                    rezultList.append(syncItem.GetRowStatussInfo())

                processedCount = processedCount + 1
                processedCountOperation = processedCountOperation + 1

                if processedCountOperation >= 1000:
                    if self.isTool:
                        self.AddMessage(u'>>>> ...Processed [{0}] from [{1}], stored [{2}], faulty [{3}] ({4})'.format(processedCount, recordCount, updatedCount, errCount, T.GetTimeReset()))
                    processedCountOperation = 0
                    if self.startOperation:
                        self.edit.stopOperation()
                    if self.startOperation:
                        self.edit.startOperation()

        self.AddMessage(u'>>>> ...Processed [{0}], stored [{1}], faulty [{2}]'.format(processedCount, updatedCount, errCount))
        return rezultList

    def __SyncDestinationToSource(self):
        """Procedure performs syncronization of type Destination To Source

        Args:
            self: The reserved object 'self'
       
        Returns:
            * output - list of dictionary items containig:
                - id: record id, 
                - syncResult: notInitialized,  synchronized, inserted, error
                - error: error mesage or '',
                - updated: true if inserted or updated
        """
        # get summary record count
        recordCount = 0
        with self.gp.da.SearchCursor(self.definition.outTable, self.definition.outTableFields, self.definition.outTableQuery) as cur:
             for row in cur:
                recordCount = recordCount + 1
        self.AddMessage(u'>>>> ...Records to process {0}'.format(recordCount))

        rezultList =[]

        updatedCount = 0
        processedCount = 0
        processedCountOperation = 0
        errCount = 0
        
        with self.gp.da.UpdateCursor(self.definition.outTable, self.definition.outTableFields, self.definition.outTableQuery) as outCur:
            T = TimerHelper.TimerHelper()
            for outRow in outCur:
                syncItem = SyncItem2()
                syncItem.outSyncRow = outRow
                syncItem.id = outRow[0]

                with self.gp.da.SearchCursor(self.definition.inTable, self.definition.inTableFields, u'{0} = {2}{1}{2}'.format(self.definition.inTableJoinField, syncItem.id, self.definition.idvalueseparator)) as inCur:
                    recordsFoundCount = 0
                    for inRow in inCur:
                        #print "rec {0} - {1}".format(u'{0} = {2}{1}{2}'.format(self.definition.outTableJoinField, syncItem.id, self.definition.idvalueseparator), (recordsFoundCount == 0 or self.definition.allowMultiple)) 
                        if recordsFoundCount == 0:
                            recordsFoundCount = recordsFoundCount + 1
                            syncItem.inSyncRow = inRow
                            syncItem.DoSyncFields()
                            if syncItem.updated:
                                outCur.updateRow(syncItem.outSyncRow)
                                updatedCount = updatedCount + 1
                                syncItem.ClerRowInfo()
                                syncItem.syncResult = 'synchronized'
                                rezultList.append(syncItem.GetRowStatussInfo())
                        else:
                            syncItem = SyncItem2()
                            syncItem.outSyncRow = outRow
                            syncItem.id = outRow[0]
                            errCount = errCount + 1
                            syncItem.error =  u"Error: row in the source table is not unique " + self.definition.messageDefinition.format(*syncItem.outSyncRow)
                            self.AddMessage(u'        ...{0}'.format(syncItem.error))
                            syncItem.ClerRowInfo()
                            syncItem.syncResult = 'error'
                            rezultList.append(syncItem.GetRowStatussInfo())

                if recordsFoundCount == 0:
                    errCount = errCount + 1
                    syncItem.error =  u"Error: no related records found in the source table " + self.definition.messageDefinition.format(*syncItem.outSyncRow)
                    self.AddMessage(u'        ...{0}'.format(syncItem.error))
                    syncItem.ClerRowInfo()
                    syncItem.syncResult = 'error'
                    rezultList.append(syncItem.GetRowStatussInfo())

                processedCount = processedCount + 1
                processedCountOperation = processedCountOperation + 1

                if processedCountOperation >= 1000:
                    if self.isTool:
                        self.AddMessage(u'>>>> ...Processed [{0}] from [{1}], stored [{2}], faulty [{3}] ({4})'.format(processedCount, recordCount, updatedCount, errCount, T.GetTimeReset()))
                    processedCountOperation = 0
                    if self.startOperation:
                        self.edit.stopOperation()
                    if self.startOperation:
                        self.edit.startOperation()

        self.AddMessage(u'>>>> ...Processed [{0}], stored [{1}], faulty [{2}]'.format(processedCount, updatedCount, errCount))
        return rezultList


class SyncItem2(object):
    """class for storing sysnc data item"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """
        self.inSyncRow = () # in sync row
        self.outSyncRow = () # out sync row
        self.syncResult = 'notInitialized' # sync state notInitialized, error, ok
        self.error = '' # sync error message
        self.id = '' # sync id
        self.updated = False # if row had changes and needed to be saved then True

    def ClerRowInfo(self):
        """Clears savec row objects"""
        del self.inSyncRow
        del self.outSyncRow

    def GetRowStatussInfo(self):
        """Gets Dict of row statuss reprezenting objects"""
        return {
                "id": self.id, 
                "syncResult": self.syncResult, 
                "error": self.error,
                "updated": self.updated
                }

    def DoSyncFields(self):
        """Procedure performs the field synchronization

        Args:
            self: The reserved object 'self'
            inRow: Row to synchronize
            outRow: Row to which synchronize

        Returns:
            * output - 0, If no changes were necessary; 1, If there were any changes
            * outRow - Altered row
        """
        inRow = self.inSyncRow
        outRow = self.outSyncRow
        for x in range(0, len(inRow)):
            if inRow[x] != outRow[x]:
                outRow[x] = inRow[x]
                self.updated = True
        self.outSyncRow = outRow


class SyncDefinition2(object):
    """Synchronization definition description class"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """
        self.inTable = "" # input Tale - if used inTableObject will not be processed
        self.inTableFields = () # tuple of input table fields - will be ignored if input has inTableObject 
        self.inTableJoinField = "" # Field for input table to be used in relation - will be ignored if input has inTableObject 
        self.inTableQuery = None # query for input table - will be ignored if input has inTableObject 
        self.inTableObject = list() # List of tuple objects reprezenting input table rows - to use do not specify inTable and inTableFields
        self.outTable = "" # destination table
        self.outTableFields = () # tuple of output table fields
        self.outTableQuery = None # query for output table (used only in Destination To Source Sync mode)
        self.outTableJoinField = "" # Field for ouput table to be used in relation
        self.messageDefinition = "" # Output message formatting
        self.idvalueseparator = "" # "'"for Text fields
        self.createNew = False # whether to create new record if input row match has not been found (used only in Source To Destination Sync mode)
        self.allowMultiple = False # whhether it is okay to sync more than one source roe to multiple rows in destination (used only in Source To Destination Sync mode)
        self.mode = "Source To Destination" # Source To Destination or Destination To Source. Sync mode that specifies sync row search type. Sync rows found in destination then search for coresponding rows in source (Destination To Source) or find all rows in source and sync with coresponding rows in destination (Source To Destination).

    def HasInTableQuery(self):
        """Returns whether input table has query"""
        hasInQuery = False
            
        if self.inTableQuery != None:
            hasInQuery = True

        return hasInQuery

    def HasOutTableQuery(self):
        """Returns whether input table has query"""
        hasInQuery = False
            
        if self.outTableQuery != None:
            hasInQuery = True

        return hasInQuery

    def SourceToDestination(self):
        """Returns whether sync mode is Source To Destination"""
        isStoD = False
            
        if self.mode.upper() == "Source To Destination".upper():
            isStoD = True

        return isStoD


# -*- coding: utf-8 -*-
"""
     Module for operations with ArcGIS Server services
"""

# For Http calls
import httplib, urllib, json, urllib2, sys
import MyError

class AGSServerHelper(object):

    def __init__(self, username, password, serverName, serverPort=6080, Tool=None, https=False):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            username: ArcGIS Server administrator username
            password: ArcGIS Server administrator password
            serverName: Server adress
            serverPort: Server port (optional)
            Tool: GISPython tool (optional)
        """
        self.username = username
        self.password = password
        self.serverName = serverName
        self.serverPort = serverPort
        self.Tool = Tool
        self.https = https
        if https:
            self.httstring = 'https'
        else:
            self.httstring = 'http'
        # Get a token
        self.token = self.genToken(username, password, serverName, serverPort)
        if self.token == "":
            raise MyError.MyError("Could not generate a token with the username and password provided.")

    def StartService(self, folder, service):
        self.StartStopService(folder, service, "START")

    def StopService(self, folder, service):
        self.StartStopService(folder, service, "STOP")

    def StartStopService(self, folder, service, action):
        # Construct URL to read folder
        if folder == None:
            folder = "ROOT"
        if folder.upper() == "ROOT":
            folder = ""
        else:
            folder += "/"

        # Define service URL with action (START/STOP)
        folderURL = "/arcgis/admin/services/" + folder + service + "/" + action

        # This request only needs the token and the response formatting parameter
        params = urllib.urlencode({'token': self.token, 'f': 'json'})

        headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}

        # Connect to URL and post parameters
        if self.https:
            httpConn = httplib.HTTPSConnection(self.serverName, self.serverPort)
        else:
            httpConn = httplib.HTTPConnection(self.serverName, self.serverPort)
        httpConn.request("POST", folderURL, params, headers)

        # Read response
        response = httpConn.getresponse()
        if response.status != 200:
            httpConn.close()
            raise MyError.MyError("Could not read folder information.")
        else:
            data = response.read()

            # Check that data returned is not an error object
            if not self.assertJsonSuccess(data):
                raise MyError.MyError("Error when reading folder information. " + str(data))
            else:
                if self.Tool != None:
                    self.Tool.AddMessage("Service " + folder + service + ' ' + action + ' done successfully ...')

            # Deserialize response into Python object
            #dataObj = json.loads(data)
            httpConn.close()

    def genToken(self, adminUser, adminPass, server, port, expiration=60):
        """Create ArcGIS server connection file

        Args:
            server: Server name
            port: Server port
            adminUser: Username
            adminPass: Password
        """
        query_dict = {'username':   adminUser,
                      'password':   adminPass,
                      'expiration': expiration,
                      'client':     'requestip'}

        query_string = urllib.urlencode(query_dict)
        url = "{}://{}:{}/arcgis/admin/generateToken".format(self.httstring, server, port)

        token = json.loads(urllib.urlopen(url + "?f=json", query_string).read())

        if "token" not in token:
            raise MyError.MyError(token['messages'])
            sys.exit()
        else:
            return token['token']

    def getServiceList(self, server, port, adminUser, adminPass, token=None):
        """Retrieve ArcGIS server services

        Args:
            self: The reserved object 'self'
            server: Server name
            port: Server port
            adminUser: Username
            adminPass: Password
            token: Token (if created)
        """
        if token is None:
            token = self.genToken(adminUser, adminPass, server, port)

        services = []
        folder = ''
        URL = "{}://{}:{}/arcgis/admin/services{}?f=pjson&token={}".format(self.httstring, server, port, folder, token)

        try:
            serviceList = json.loads(urllib2.urlopen(URL).read())
        except urllib2.URLError, e:
            raise MyError.MyError(e)

        # Build up list of services at the root level
        for single in serviceList["services"]:
            services.append(single['serviceName'] + '.' + single['type'])

        # Build up list of folders and remove the System and Utilities folder (we dont want anyone playing with them)
        folderList = serviceList["folders"]
        folderList.remove("Utilities")
        folderList.remove("System")

        if len(folderList) > 0:
            for folder in folderList:
                URL = "{}://{}:{}/arcgis/admin/services/{}?f=pjson&token={}".format(self.httstring, server, port, folder, token)
                fList = json.loads(urllib2.urlopen(URL).read())

                for single in fList["services"]:
                    services.append(folder + "//" + single['serviceName'] + '.' + single['type'])

        if len(services) == 0:
            if self.Tool != None:
                self.Tool.AddMessage("No services found")
        else:
            if self.Tool != None:
                self.Tool.AddMessage("Services on " + server +":")
            for service in services:
                statusURL = "{}://{}:{}/arcgis/admin/services/{}/status?f=pjson&token={}".format(self.httstring, server, port, service, token)
                status = json.loads(urllib2.urlopen(statusURL).read())
                if self.Tool != None:
                    self.Tool.AddMessage("  " + status["realTimeState"] + " > " + service)

        return services

    def GetServerJson(self, token, serverName, serverPort, serverService):
        """Retrieve service parameters

        Args:
            self: The reserved object 'self'
            token: Token
            serverName: Server name
            serverPort: Server port
            serverService: Service which parameter configuration shall be retrieved
        """
        # This request only needs the token and the response formatting parameter
        params = urllib.urlencode({'token': token, 'f': 'json'})

        headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
        serviceURL = "/arcgis/admin/services/" + serverService

        # Connect to service to get its current JSON definition
        if self.https:
            httpConn = httplib.HTTPSConnection(serverName, serverPort)
        else:
            httpConn = httplib.HTTPConnection(serverName, serverPort)
        httpConn.request("POST", serviceURL, params, headers)

        # Read response
        response = httpConn.getresponse()
        if response.status != 200:
            httpConn.close()
            raise MyError.MyError(u'...Couldn\'t retrieve service parameter configuration\n')
            return
        else:
            data = response.read()

            # Check that data returned is not an error object
            if not self.assertJsonSuccess(data):
                raise MyError.MyError(u'...Couldn\'t retrieve service parameter configuration: ' + str(data) + '\n')
            else:
                if self.Tool != None:
                    self.Tool.AddMessage(u'...Service parameter configuration successfully retrieved\n')

        dataObj = json.loads(data)
        httpConn.close()
        return dataObj

    def PublishServerJson(self, service, serverName, dataObj, token, serverPort):
        """Publish service parameters to server

        Args:
            self: The reserved object 'self'
            service: Service which parameter configuration shall be renewed
            serverName: Server name
            dataObj: Parameter configuration
            token: Token
            serverPort: Server port
        """
        if self.https:
            httpConn = httplib.HTTPSConnection(serverName, serverPort)
        else:
            httpConn = httplib.HTTPConnection(serverName, serverPort)
        headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}

        # Serialize back into JSON
        updatedSvcJson = json.dumps(dataObj)

        # Call the edit operation on the service. Pass in modified JSON.
        editSvcURL = "/arcgis/admin/services/" + service + "/edit"
        params = urllib.urlencode({'token': token, 'f': 'json', 'service': updatedSvcJson})
        httpConn.request("POST", editSvcURL, params, headers)

        # Read service edit response
        editResponse = httpConn.getresponse()
        if editResponse.status != 200:
            httpConn.close()
            if self.Tool != None:
                self.Tool.AddMessage(u'...Service configuration renewal error\n')
            return
        else:
            editData = editResponse.read()

            # Check that data returned is not an error object
            if not self.assertJsonSuccess(editData):
                if self.Tool != None:
                    self.Tool.AddMessage(u'...Service configuration renewal error: ' + str(editData) + '\n')
            else:
                if self.Tool != None:
                    self.Tool.AddMessage(u'...Service configuration succesfully renewed\n')

        httpConn.close()

        return

    def getServiceFromServer(self, services, service, serviceDir):
        """Retrieve the full service name from the server

        Args:
            self: The reserved object 'self'
            services: List of all services on server
            service: Name of the service from which to get corresponding name from the server services list
            serviceDir: Name of the service directory which is shown in the configuration of services to be published on the server
        """
        if serviceDir == None:
            configService = service
        else:
            configService = serviceDir + "//" + service
        for serverService in services:
            if serverService.split('.')[0].upper() == configService.upper():
                return serverService
            else:
                serverService = ''

        return serverService

    def assertJsonSuccess(self, data):
        """A function that checks that the input JSON object is not an error object.

        Args:
            self: The reserved object 'self'
            data: JSON data object
        """
        obj = json.loads(data)
        if 'status' in obj and obj['status'] == "error":
            raise MyError.MyError("Error: JSON object returns an error. " + str(obj))
            return False
        else:
            return True

    #Check if service is running
    def IsServiceRunning(self, folder, service):
        """Retrieve the service status from the server

        Args:
            self: The reserved object 'self'
            folder: Service directory
            service: Name of a service
        """
        # Construct URL to read folder
        if folder == None:
            folder = "ROOT"
        if folder.upper() == "ROOT":
            folder = ""
        else:
            folder += "/"

        folderURL = "/arcgis/admin/services/" + folder

        # This request only needs the token and the response formatting parameter
        params = urllib.urlencode({'token': self.token, 'f': 'json'})

        headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}

        # Connect to URL and post parameters
        if self.https:
            httpConn = httplib.HTTPSConnection(self.serverName, self.serverPort)
        else:
            httpConn = httplib.HTTPConnection(self.serverName, self.serverPort)

        httpConn.request("POST", folderURL, params, headers)

        # Read response
        response = httpConn.getresponse()
        if response.status != 200:
            httpConn.close()
            raise MyError.MyError("Could not read folder information.")
        else:
            data = response.read()

        # Check that data returned is not an error object
        if not self.assertJsonSuccess(data):
            raise MyError.MyError("Error while reading folder information. " + str(data))

        # Deserialize response into Python object
        dataObj = json.loads(data)

        httpConn.close()

        # Construct URL to stop or start service, then make the request
        statusURL = "/arcgis/admin/services/" + folder + service + "/status"
        httpConn.request("POST", statusURL, params, headers)

        # Read status response
        statusResponse = httpConn.getresponse()
        if statusResponse.status != 200:
            httpConn.close()
            raise MyError.MyError("Error while checking status for " + service)
            return
        else:
            statusData = statusResponse.read()

            # Check that data returned is not an error object
            if not self.assertJsonSuccess(statusData):
                raise MyError.MyError("Error while retrieving status information for " + service + ".")
            else:
                # Get service status
                statusDataObj = json.loads(statusData)
                if statusDataObj['realTimeState'] == "STOPPED":
                    return False #print "Service " + service + " was detected to be stopped"
                else:
                    return True #print "Service " + service + " is running"

            httpConn.close()



# -*- coding: utf-8 -*-
"""
     Module defines abstract classes for the ESRI Toolbox tool definition
"""

class GISPythonTool(object):
    """Class which helps to create an ArcGIS Toolbox"""
    def __init__(self):
        """Define the tool (tool name is the class name)"""
        self.canRunInBackground = False

    def getParameterInfo(self):
        """Define parameter definitions"""
        params = None
        return params

    def isLicensed(self):
        """Determines if the tool is licenced for execution"""
        return True

    def updateParameters(self, parameters):
        """This method is called in case the parameters are changed,
        and is used for setting up the parameters
        """
        validator = getattr(self, 'ToolValidator', None)
        if validator:
            return validator(parameters).updateParameters()

    def updateMessages(self, parameters):
        """This method is called after an inner validation,
        and is intended for carrying out an additional validations
        """
        validator = getattr(self, 'ToolValidator', None)
        if validator:
            return validator(parameters).updateMessages()

    def execute(self, parameters, messages):
        """Executes a tool"""
        raise NotImplementedError

class ToolValidator(object):
    """Class for validating the tool's parameter values and controlling
    the behavior of the tool's dialog.
    """
    def __init__(self, parameters):
        """Setup arcpy and the list of the tool's parameters."""
        self.params = parameters

    def initializeParameters(self):
        """Refine properties of the tool's parameters. This method is
        called when the tool is opened.
        """
        return

    def updateParameters(self):
        """Modify the values and properties of parameters before internal
        validation is performed. This method is called whenever a parameter
        has been changed.
        """
        return

    def updateMessages(self):
        """Modify the messages created by internal validation for each tool
        parameter. This method is called after internal validation.
        """
        return



# -*- coding: utf-8 -*-
"""Base class for GISPython Tool object
"""

import os
import datetime
import sys
import codecs
import subprocess
import shlex
import shutil
import locale
import time
import string
import unicodedata
import uuid
from multiprocessing import Process, Queue

class GISPythonToolBase(object):
    def __init__(self, ToolName, Params):
        self.ExecutePatch = os.path.dirname(os.path.realpath(__file__))
        self.Pr = Params
        self.StartTime = datetime.datetime.now()
        self.ToolName = ToolName
        self.OutputStr = u''
        self.OutputErrStr = u''
        self.State = "Started"
        
        if hasattr(Params, 'encodingPrimary'):
            UnicodeWriter = codecs.getwriter(Params.encodingPrimary)
            sys.stdout = UnicodeWriter(sys.stdout)

    def MyEnd(self):
        """End of the process

        Args:
            self: The reserved object 'self'
        """
        self.AddMessage(u'End tool ' + self.ToolName + u' execution ' + self.MyNow())
        TD = datetime.datetime.now()- self.StartTime
        self.AddMessage(u'      ...Tool execution time: ' + str(TD))
        self.State = "Done"

    def MyDispose(self):
        """Disposal of the class

        Args:
            self: The reserved object 'self'
        """
        if self.State == "Started":
            self.MyEnd()
        self.State = "Disposed"

    def AddMessage(self, strMessage, newline=True):
        """Procedure for a message output (screen, logfile and if necessary e-mail)

        Args:
            self: The reserved object 'self'
            strMessage: Output message text
            newline: Flag that marks that the output must be continued in a new line
        """
        if strMessage == None:
            strMessage = ''
        if not isinstance(strMessage, unicode):
            if not isinstance(strMessage, str):
                strMessage = str(strMessage)
        if strMessage[-1:] == '\n':
            strMessage = strMessage[0:-1]

        print strMessage

        self.OutputStr += strMessage
        if newline == True:
            self.OutputStr += '\n'

    def AddError(self, strMessage, newline=True):
        """Procedure for the GP object error message output (screen, logfile and if necessary e-mail)

        Args:
            self: The reserved object 'self'
            strMessage: Output message text
        """
        if strMessage == None:
            strMessage = ''
        if not isinstance(strMessage, unicode):
            if not isinstance(strMessage, str):
                strMessage = str(strMessage)
        if strMessage[-1:] == '\n':
            strMessage = strMessage[0:-1]

        self.AddMessage('ERROR> ' + strMessage, newline)

        self.OutputErrStr += strMessage
        if newline == True:
            self.OutputErrStr += '\n'
        
    def AddWarning(self, strMessage, newline=True):
        """Procedure for the GP object warning message output (screen, logfile and if necessary e-mail)

        Args:
            self: The reserved object 'self'
            strMessage: Output message text
        """
        if strMessage == None:
            strMessage = ''
        if not isinstance(strMessage, unicode):
            if not isinstance(strMessage, str):
                strMessage = str(strMessage)
        if strMessage[-1:] == '\n':
            strMessage = strMessage[0:-1]

        self.AddMessage('Warning> ' + strMessage, newline)

        self.OutputErrStr += strMessage
        if newline == True:
            self.OutputErrStr += '\n'

    def _tryCovertStringEncoding(self, txt):
        """Function for working with strings in diferent encodings. Converts string from input to string in correct encoding.

        Args:
            self: The reserved object 'self'
            txt: String to be converted

        Returns:
            converted string
        """
        if isinstance(txt, str):
            try:
                txt = txt.decode(self.Pr.encodingPrimary, errors='strict')
            except Exception:
                try:
                    txt = txt.decode(self.Pr.encodingSecondary, errors='strict')
                except Exception:
                    try:
                        txt = txt.decode('utf-8', errors='strict')
                    except Exception:
                        txt = txt.decode(self.Pr.encodingPrimary, errors='replace')
        return txt

    def CorrectStr(self, Str):
        """Function doubles symbol '\' in path for some external execution compatibility
        Args:
            self: The reserved object 'self'
            Str: Input string

        Returns:
            string
        """
        return Str.replace('\\', '\\\\')

    def AchiveFiles(self, Dir, AchiveDir, FileName, PrintOut=True):
        """Function moves log files to the archive

        Args:
            self: The reserved object 'self'
            Dir: Directory from which to archive
            AchiveDir: Archive directory
            FileName: Parameter for searching a file (full or partial filename)
        """
        # Check if Archive directory exists
        if os.path.exists(AchiveDir):
            names = os.listdir(Dir)
            for name in names:
                if name.lower().find(FileName.lower()) == 0:
                    if PrintOut:
                        print("        move: "  + Dir + "\\" + name + " -> " + AchiveDir + "\\" + name + "\n")
                    try:
                        shutil.move(Dir + "\\" + name, AchiveDir + "\\" + name)
                    except:
                        print("        archiving failed : "  + Dir + "\\" + name + " -> " + AchiveDir + "\\" + name + "\n")

    def MyNowFile(self):
        """Function returns formatted date for the filename output

        Args:
            self: The reserved object 'self'

        Returns:
            Date, formatted as text
        """
        return datetime.datetime.strftime(datetime.datetime.now(), "%Y-%m-%d_%H-%M-%S")

    def MyNowOracle(self):
        """Function returns formatted date for the data selection in SQL

        Args:
            self: The reserved object 'self'

        Returns:
            Date, formatted as text
        """
        return "TO_DATE('" + datetime.datetime.strftime(datetime.datetime.now(), "%Y-%m-%d %H:%M:%S") + "', 'YYYY-MM-DD HH24:MI:SS')"

    def MyNowFileSafe(self):
        """Function returns formatted date for the filename output (additional compatibility)

        Args:
            self: The reserved object 'self'

        Returns:
            Date, formatted as text
        """
        return datetime.datetime.strftime(datetime.datetime.now(), "%Y%m%d_%H%M%S")

    def MyNow(self):
        """Function returns formatted date for the output

        Args:
            self: The reserved object 'self'

        Returns:
            Date, formatted as text
        """
        return datetime.datetime.strftime(datetime.datetime.now(), "%Y-%m-%d %H:%M:%S")

    def MyNowUTC(self):
        """Function returns formatted date for the UTC date output

        Args:
            self: The reserved object 'self'

        Returns:
            Date, formatted as text
        """
        return datetime.datetime.strftime(datetime.datetime.utcnow(), "%Y-%m-%d %H:%M:%S")

    def MyNowForParam(self, minusdays=0, includeTime = False):
        """Function returns formatted date (date now) for the GEO parameter processing

        Args:
            self: The reserved object 'self'
            minusdays: Number of days to subtract from today
            includeTime: Include time in param (True/False)

        Returns:
            Date, formatted as text
        """
        if includeTime:
            DD = datetime.timedelta(days=minusdays)
            return datetime.datetime.strftime((datetime.datetime.now() - DD), "%Y-%m-%d %H:%M:%S")
        else:
            DD = datetime.timedelta(days=minusdays)
            return datetime.datetime.strftime((datetime.datetime.now() - DD), "%Y-%m-%d")

    def MyDateFromParam(self, dateString, includeTime = False):
        """Function converts date written in the parameter file to a date object

        Args:
            self: The reserved object 'self'
            includeTime: Include time in param (True/False)
        """
        if includeTime:
            return datetime.datetime.strptime(dateString, "%Y-%m-%d %H:%M:%S")
        else:
            return datetime.datetime.strptime(dateString, "%Y-%m-%d")

    def MyDateForParam(self, paramStr, includeTime = False):
        """Function returns date from GEO parameter processing saved string

        Args:
            self: The reserved object 'self'
            paramStr: Parameter value as text
            includeTime: Include time in param (True/False)

        Returns:
            datetime
        """
        if includeTime:
            return datetime.datetime.strptime(paramStr, "%Y-%m-%d %H:%M:%S")
        else:
            return datetime.datetime.strptime(paramStr, "%Y-%m-%d")

    def _runProcess(self, exe, noErr=False, Detached=False, Silent=False, hidenStrings = []):
        """Shell command execution support function (see the runShell function)

        Args:
            self: The reserved object 'self'
            exe: Executable command

        Return:
            stdoutdata
        """
        lines = list()
        if Detached == True:
            DETACHED_PROCESS = 0x00000010
            p = subprocess.Popen(exe, creationflags=DETACHED_PROCESS, stdout=None, stderr=None, shell=True, universal_newlines=True, close_fds=True) #, env={'LANG':'C'})
        else:
            p = subprocess.Popen(exe, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, universal_newlines=True) #, env={'LANG':'C'})
            while True:
                retcode = p.poll() #returns None while subprocess is running
                str = p.stdout.readline()
                for hideString in hidenStrings:
                    str = str.replace(hideString, '*' * len(hideString))
                if str != '' and Silent != True:
                    self.AddMessage(u'>>>>' + self._tryCovertStringEncoding(str), True)
                lines.append(str)
                if retcode is not None:
                    break
            errlines = p.stderr.readlines()
            if not errlines == None:
                for str in errlines:
                    line = self._tryCovertStringEncoding(str)
                    for hideString in hidenStrings:
                        line = line.replace(hideString, '*' * len(hideString))
                    if line != '':
                        if noErr == False:
                            self.AddError('>>>>' + line, True)
                        else:
                            self.AddMessage('>>>>' + line, True)
        return lines

    def runShell(self, exe, noErr=False, ErrorStrings=['ERROR', 'FAILED', u'KĻŪDA', 'EXCEPTION', 'ORA-'], Detached=False, Silent=False, hidenStrings = []):
        """Shell command execution procedure. It can detect errors in execution and can output results to screen.

        Args:
            self: The reserved object 'self'
            exe: Executable command
            noErr: 'True' indicates that the errors doesn't have to be logged
            ErrorStrings: List of error strings to look for in the output. Found error will be considered as an error in the execution process
            Detached: Whether to execute seperately from the main process (Default: False)
            hidenStrings: List of strings tha has to be hiden in the output (used for hiding passwords)
        """
        if not isinstance(exe, list):
            args = shlex.split(exe)
        else:
            args = exe
            exe = " ".join(exe)
        _StartTime = datetime.datetime.now()
        if not Silent:
            command = exe
            for hideString in hidenStrings:
                command = command.replace(hideString, '*' * len(hideString))
            self.AddMessage(u'>Executing the Shell command> ' + command)
        ShellOutput = self._runProcess(args, noErr, Detached, Silent, hidenStrings)
        lines = ShellOutput
        self._outputLines(lines, False, noErr, ErrorStrings, Silent, hidenStrings = hidenStrings)
        _TD = datetime.datetime.now()- _StartTime
        if not Silent:
            self.AddMessage(u'>Done executing the Shell command. Execution time '  + str(_TD))

    def outputLogfile(self, file, encoding='utf8', noErr=False, ErrorStrings=['ERROR', 'FAILED', u'KĻŪDA', 'EXCEPTION', 'ORA-'], Silent=False, hidenStrings = []):
        """Procedure prints text file to screent - processing error keywords

        Args:
            self: The reserved object 'self'
            file: path to file to process
            noErr: True ir no error logging is necesery
            ErrorStrings: List or keywords with will be recognized as errors
            Silent: if True no Errors will be rised
            hidenStrings: List of strings tha has to be hiden in the output (used for hiding passwords)
        """
        with codecs.open(file, 'r', encoding) as fin:
            self._outputLines(fin.readlines(), True, noErr, ErrorStrings, Silent, hidenStrings = hidenStrings)

    def _outputLines(self, lines, doMessges, noErr=False, ErrorStrings=['ERROR', 'FAILED', u'KĻŪDA', 'EXCEPTION', 'ORA-'], Silent=False, hidenStrings = []):
        """Procedure for outputing set of lines to screen with error key word recognition. (for example for log file output processing)

        Args:
            self: The reserved object 'self'
            lines: Lines to process
            noErr: True ir no error logging is necesery
            ErrorStrings: List or keywords with will be recognized as errors
            Silent: if True no Errors will be rised
            hidenStrings: List of strings tha has to be hiden in the output (used for hiding passwords)
        """
        isError = False
        for line in lines:
            for hideString in hidenStrings:
                line = line.replace(hideString, '*' * len(hideString))
            line = self._tryCovertStringEncoding(line)
            if line != '':
                if doMessges == True and Silent==False:
                    self.AddMessage(line)
                if noErr == False:
                    for ErStr in ErrorStrings:
                        if line.upper().find(ErStr.upper()) > -1:
                            isError = True
        for line in lines:
            for hideString in hidenStrings:
                line = line.replace(hideString, '*' * len(hideString))
            line = self._tryCovertStringEncoding(line)
            if line != '':
                if isError == True:
                    self.AddWarning('>>>>' + line)

    def AutorizeNTWLocation(self, Adress, user, pwd):
        """Network directory authorization

        Args:
            self: The reserved object 'self'
            Adress: Network adress
            user: Username
            pwd: Password
        """
        self.runShell(self.CorrectStr(r"net use " + Adress + r" /delete /y"), True, [])
        self.runShell(self.CorrectStr(r'net use ' + Adress + r' "' + pwd + r'" /user:' + user), True, [])

    def GetSQL(self, Name):
        """Function gets the SQL file location from the installation directory

        Args:
            self: The reserved object 'self'
            Name: Filename without an extension

        Returns:
            SQL file full path
        """
        return self.ExecutePatch + '\\SQL\\' + Name + '.sql'

    def RunSQL(self, Name, user="#", pwd="#", SpoolFile='#', ErrorStrings=['ERROR', 'FAILED', u'KĻŪDA', 'EXCEPTION', 'ORA-'], params=[], DBType='Oracle', hidenStrings = [], DBName='#'):
        """Procedure for SQL file execution (only Oracle sqlplus supported)
        Typically used for execution, passing only SQL filename parameter

        Args:
            self: The reserved object 'self'
            Name: Filename without an extension
            u: Username
            p: Password
            SpoolFile: SQL output
            ErrorStrings: A list of keyword error strings that defines an error in the execution
            params: aditional parameters
            DBType: only Oracle is supported
            hidenStrings: List of strings tha has to be hiden in the output (used for hiding passwords)
            DBName: Oracle TNS name
        """
        if DBType.upper() == 'ORACLE':
            if pwd == "#":
                pwd = self.Pr.p
            if user == "#":
                user = self.Pr.u
            if DBName == "#":
                DBName = self.Pr.TNSName
            if SpoolFile == '#':
                SpoolFile = self.Pr.OutDir + '\\' + Name + self.MyNowFile() + 'SQL.outlog'
            hidenStrings.append(pwd)
            self.AchiveFiles(self.Pr.OutDir, self.Pr.OutDirArh, Name, False)
            self.runShell('sqlplus -L ' + user + '/' + pwd + '@' + DBName + ' @"' + self.GetSQL(Name) + '" "' + SpoolFile + '" ' + ' '.join(params), False, ErrorStrings, hidenStrings = hidenStrings)
        else:
            raise AttributeError('Provided DB Type is not supported!')

    def GetPS(self, Name, Path='#'):
        """Function gets the PowerShell file location from the installation directory

        Args:
            self: The reserved object 'self'
            Name: Filename without an extension
        """
        if Path == '#':
            self.ExecutePatch
        return Path + '\\PShell\\' + Name + '.ps1'

    def RunPS(self, Name, Args, Path='#'):
        """Procedure for the PowerShell file execution

        Args:
            self: The reserved object 'self'
            Name: Filename without an extension
            Args: Arguments
        """
        if Path == '#':
            self.ExecutePatch
        self.runShell('powershell -File "' + self.GetPS(Name, Path) + '" ' + Args)

    def run_with_limited_time(self, func, args, kwargs, time):
        """Runs a function with time limit

        Args:
            self: The reserved object 'self'
            func: The function to run
            args: The functions args, given as a tuple
            kwargs: The functions args, given as a tuple
            time: The time limit in seconds

        Returns:
            True if the function ended successfully. False if it was terminated.
        """
        p = Process(target=func, args=args, kwargs=kwargs)
        p.start()
        p.join(time)
        if p.is_alive():
            p.terminate()
            return False
        return True



# -*- coding: utf-8 -*-
"""
     GDB operations module
"""
import os
import uuid

class GDBHelper:
    """Class for easing the ESRI geodatabase operations"""

    def __init__(self, gp, Tool=None):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            gp: ArcPy GP object
        """
        self.gp = gp
        if not Tool == None:
            self.isTool = True
        else:
            self.isTool = False
        self.Tool = Tool

    def AddWarning(self, str):
        if self.isTool:
            self.Tool.AddWarning(str)
        else:
            self.gp.AddWarning(str)

    def OutputMessages(self):
        if self.isTool:
            self.Tool.OutputMessages(str)


    def GetDSConnectedElements(self, ConnDBSchema, DBName, IncludeMe=False):
        """Function defines all the tables which are linked with the FeatureDataset

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to DB schema
            DBName: DB name to process
            IncludeMe: Optional. Default = False. Indicates if DB classes will be returned in the returned list

        Returns:
            * Result list with the found object classes
        """
        self.gp.env.workspace = ConnDBSchema + '\\' + DBName
        fcs = self.gp.ListFeatureClasses()
        DSfcs = fcs
        ResultList = list()
        if DSfcs != None:
            for fc in fcs:
                self.gp.AddMessage(u'            ... searching relations ' + fc)
                dfc = self.gp.Describe(fc)
                ResultList = list(set(self.GetRelations(ConnDBSchema, dfc, ResultList) + ResultList))
            for x in DSfcs:
                if x in ResultList:
                    ResultList.remove(x)
            if IncludeMe == True:
                DSfcs2 = list()
                for x in DSfcs:
                    DSfcs2.append(DBName + '\\' + x)
                ResultList = list(set(ResultList + DSfcs2))
        return ResultList

    def GetRelations(self, ConnDBSchema, dfc, existingObjects):
        """Auxiliary function for 'GetDSConnectedElements' function

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            dfc: Describe 'FeatureClass' object
            existingObjects: List of existing objects

        Returns:
            List of the found objects
        """
        existingObjects.append(dfc.name)
        rcs = dfc.relationshipClassNames
        ResultList = list()
        for rc in rcs:
            drc = self.gp.Describe(ConnDBSchema + '\\' + rc)
            if drc.dataType == "RelationshipClass":
                origins = drc.originClassNames
                destinations = drc.destinationClassNames
                ResultList = list(set(ResultList + origins))
                ResultList = list(set(ResultList + destinations))
            else:
                self.gp.AddWarning(u'        ...Descriptive object for relational class [' + ConnDBSchema + '\\' + rc + u'] not found')
        for x in existingObjects:
            if x in ResultList:
                ResultList.remove(x)
        for x in ResultList:
            dfc2 = self.gp.Describe(ConnDBSchema + '\\' + x)
            existingObjects = list(set(existingObjects + self.GetRelations(ConnDBSchema, dfc2, existingObjects)))
        return existingObjects

    def CalculateXY(self, Layer, Field, type, Query):
        """Function for calculating the fields X and Y

        Args:
            self: The reserved object 'self'
            Layer:  Input layer
            Field: The field to be calculated
            type: X or Y
            Query: Layer query
        """
        xExpression = "getcord('X', !SHAPE.CENTROID!)"
        yExpression = "getcord('Y', !SHAPE.CENTROID!)"
        codeBlock = "def getcord(typ, Cent=''):\\n    Cent = Cent.replace('\\\"', '')\\n    if typ == 'X':\\n        if not Cent == '':\\n            return float(Cent.split()[0])\\n        else:\\n            return 0\\n    else:\\n        if not Cent == '':\\n            return float(Cent.split()[1])\\n        else:\\n            return 0\\n"

        CalcLayer = (self.gp.MakeTableView_management(Layer, "#" + Field, Query, "#")).getOutput(0)
        if type == 'X':
            self.gp.CalculateField_management(CalcLayer, Field, xExpression, "PYTHON", codeBlock)
        else:
            self.gp.CalculateField_management(CalcLayer, Field, yExpression, "PYTHON", codeBlock)

    def HasField(self, ConnDBSchema, Table, Field):
        """Function determines if field already exists in the table

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            Table: Table
            Field: The field to be calculated
        """
        self.gp.env.workspace = ConnDBSchema
        dt = self.gp.Describe(Table)
        fs = dt.fields
        for f in fs:
            if f.name.upper() == Field.upper():
                return True
        return False

    def GetFieldNames(self, db_object):
        """Function gets all of the field names in table

        Args:
            self: The reserved object 'self'
            db_object: Path to DB object to describe
        """
        table_description = self.gp.Describe(db_object)
        fields = table_description.fields
        field_names = [f.name for f in fields]
        return field_names

    def DelleteField(self, ConnDBSchema, TABLENAME, FIELDNAME):
        """Function eases work with the ESRI field deletion function DeleteField_management

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            Table: Table
            Field: The field to be calculated
        """
        if self.HasField(ConnDBSchema, TABLENAME, FIELDNAME):
            try:
                self.gp.DeleteField_management(os.path.join(ConnDBSchema, TABLENAME), FIELDNAME)
            except:
                self.AddWarning(u'      ...Failed to delete the field [{0}.{1}]'.format(TABLENAME, FIELDNAME))
            self.OutputMessages()
        else:
            self.AddWarning(u'      ...Cannot delete the field [{0}.{1}] which does not exist'.format(TABLENAME, FIELDNAME))

    def CreateIndex(self, ConnDBSchema, TABLENAME, FIELDNAME):
        """Function eases work with the ESRI indexing function 'AddIndex_management'

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            Table: The table
            Field: The field to be calculated
        """
        idxName = 'idx' + TABLENAME.split('.')[1][:5] + FIELDNAME[:10]
        if self.HasField(ConnDBSchema, TABLENAME, FIELDNAME):
            try:
                self.gp.AddIndex_management(os.path.join(ConnDBSchema, TABLENAME), FIELDNAME, idxName)
            except:
                self.AddWarning(u'Failed to create index [{2}] field [{0}.{1}]'.format(TABLENAME, FIELDNAME, idxName))
            self.OutputMessages()
        else:
            self.AddWarning(u'      ...Cannot create the index [{2}] for the field [{0}.{1}] because the field does not exist'.format(TABLENAME, FIELDNAME, idxName))


    def DelleteObject(self, ConnDBSchema, ObjectName, ObjectType='#'):
        """Function eases work with the ESRI object deletion function 'Delete_management'

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            ObjectName: Object to be deleted
            ObjectType: Field to be deleted
        """
        try:
            self.gp.Delete_management(os.path.join(ConnDBSchema, ObjectName), ObjectType)
        except:
            self.AddWarning(u'Failed to delete the object [' +  ObjectName + ']')
        self.OutputMessages()

    def DelleteDomain(self, ConnDBSchema, ObjectName):
        """Function eases work with the ESRI domain deletion function 'DeleteDomain_management'

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            ObjectName: The domain to be deleted
        """
        try:
            self.gp.DeleteDomain_management(ConnDBSchema, ObjectName)
        except:
            self.AddWarning(u'Failed to delete the domain [' +  ObjectName + ']')
        self.OutputMessages()

    def ClearData(self, ConnDBSchema, ObjectName, ObjectType="TABLE"):
        """Function eases feature deletion from layers and tables

        Args:
            self: The reserved object 'self'
            ConnDBSchema: Connection to the DB schema
            ObjectName: The table or layer containing features to be deleted
            ObjectType: Type: "TABLE" or "FC"
        """
        try:
            if ObjectType == 'TABLE':
                self.gp.DeleteRows_management(os.path.join(ConnDBSchema, ObjectName))
            else:
                self.gp.DeleteFeatures_management(os.path.join(ConnDBSchema, ObjectName))
        except:
            self.AddWarning(u'Failed to delete data from [' +  ObjectName + ']')
        self.OutputMessages()

    def Decode(self, val, DecodeDict):
        """Function decodes one value to another

        Args:
            self: The reserved object 'self'
            val: Value to be decoded
            DecodeDict: Python Dictionary syntax which describes the decoding process - for example {'Key1':'Val1','Key2':'Val2'}
        """
        Options = DecodeDict
        if val in Options:
            return Options[val]
        else:
            return val

    def CopyDomain(self, inWorkspace, inDomain, outWorkspace, outDomain):
        """Procedure doeas copy one domain to another.

        Args:
            self: The reserved object 'self'
            inWorkspace: in workspace
            inDomain: in domain name
            outWorkspace: out workspace
            outDomain: out workspace name
        """
        pj = os.path.join
        if self.isTool:
            self.Tool.AddMessage(u'>>>>Copy domains from {0} to {1} - {2}'.format(inDomain, outDomain, self.Tool.MyNow()))

        tableName = 'in_memory/' + inDomain
        self.gp.DomainToTable_management(inWorkspace, inDomain, tableName, 'code', 'value')
        self.OutputMessages()
        self.gp.TableToDomain_management(tableName, 'code', 'value', outWorkspace, outDomain, '#', 'REPLACE')
        self.OutputMessages()
        self.gp.Delete_management(tableName)
        self.OutputMessages()

    def DecodeField(self, Layer, Field, DecodeField, DecodeDict, Query=None, workspace=None, startEditing=False, startOperation=False, with_undo=False, multiuser=False):
        """Function does field value recalculation decoding values from one to another. Used, for example, in clasificator value recalculation.

        Args:
            self: The reserved object 'self'
            Layer: Layer in which to decode
            Field: Field in which to decode
            DecodeField: Field from which to decode
            DecodeDict: Python Dictionary syntax which describes the decoding process - for example {'Key1':'Val1','Key2':'Val2'}
            Query: Layer query
            workspace: DB in which to start data editing
            startEditing: Start the edit session in the DB specified in the 'workspace' parameter
            startEditing: Start the edit operation in the DB specified in the 'workspace' parameter
            with_undo: Sets whether the undo and redo stacks are enabled or disabled for an edit session.
            multiuser: When False, you have full control of editing a nonversioned, or versioned dataset.
        """
        if self.isTool:
            self.Tool.AddMessage(u'>>>>Executing the layer ' + uni(Layer) + u' field ' + uni(Field) + u' decoding ' + uni(DecodeDict) + u' after field ' + uni(DecodeField) + u' records after query ' + uni(Query) + ' - ' + self.Tool.MyNow())

        if startEditing or startOperation:
            edit = self.gp.da.Editor(workspace)
        if startEditing:
            edit.startEditing(with_undo, multiuser)
        if startOperation:
            edit.startOperation()

        if Field != DecodeField:
            with self.gp.da.UpdateCursor(Layer, [Field, DecodeField], Query) as cur:
                i = 0
                for row in cur:
                    row[0] = self.Decode(row[1], DecodeDict)
                    cur.updateRow(row)
                    i += 1
        else:
            with self.gp.da.UpdateCursor(Layer, [Field], Query) as cur:
                i = 0
                for row in cur:
                    row[0] = self.Decode(row[0], DecodeDict)
                    cur.updateRow(row)
                    i += 1

        if startOperation:
            edit.stopOperation()
        if startEditing:
            edit.stopEditing(True)

        if self.isTool:
            self.Tool.AddMessage(u'>>>> ...Processed  ' + str(i) + u' rows')


class RowHelper2:
    """Row processing class"""

    def __init__(self, gp, Tool=None):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            gp: ArcPy GP object
        """
        self.gp = gp
        if not Tool == None:
            self.isTool = True
        else:
            self.isTool = False
        self.Tool = Tool

    def GetUniqueValues(self, festureClass, getField, where_clause=None):
        """Get unique values from the field in the table (Only in DB)

        Args:
            self: The reserved object 'self'
            festureClass: The feature class containing the rows to be searched
            getField: The field from which to retrieve unique values
            where_clause: SQL WHERE clause to obtain the data
        """
        rezultList = list()
        with self.gp.da.SearchCursor(festureClass, field_names=(getField), where_clause=where_clause, sql_clause=('Distinct', None)) as cursor:
            for row in cursor:
                rezultList.append(row[0])
        return rezultList

    def ValidateRowsForSQLClause(self, festureClass, fields, where_clause, outStringformat, idStringformat='{0}'):
        """Validate the rows with the SQL clause

        Args:
            self: The reserved object 'self'
            festureClass: The feature class containing the rows to be searched
            fields: A list of the field names (order is important, because the parameter 'outStringformat' is configured by this parameter)
            where_clause: SQL WHERE clause to obtain the data
            outStringformat: Error output text on the found error.
                                You can use the 'unicode.format' function notation {#} to transfer the row values.
                                For example - a field with the index 0 in the list of fields with 'outStringformat' parameter value: u"faulty feature OID: {0}" will return the string: u"faulty feature OID: 123", where 123 is the 'objectid' field value for found record.
            idStringformat: Row id output text on the found error.
                            You can use the 'unicode.format' function notation {#} to transfer the row values.
        Returns:
            List containig list of two values - row id string and row error description string
        """
        rezultList = []
        with self.gp.da.SearchCursor(festureClass, field_names=fields, where_clause=where_clause) as cursor:
            for row in cursor:
                rezultList.append([unicode(idStringformat).format(*row), unicode(outStringformat).format(*row)])
        return rezultList

    def ValidateRowsForFieldValueList(self, festureClass, getField, fields, valuelist, where_clause, outStringformat, idStringformat='{0}'):
        """Check rows if the field matches the unique value list

        Args:
            self: The reserved object 'self'
            festureClass: The feature class containing the rows to be searched
            getField: Field to check
            fields: A list of field names (order is important, because parameter 'outStringformat' is configured by this parameter !!! 'getField' value should be in this list!!!
            valuelist: A list of values
            where_clause: SQL WHERE clause to obtain the data
            outStringformat: Error output text on the found error.
                                You can use the 'unicode.format' function notation {#} to transfer the row values.
                                For example - a field with the index 0 in the list of fields with 'outStringformat' parameter value: u"faulty feature OID: {0}" will return the string: u"faulty feature OID: 123", where 123 is the 'objectid' field value for found record.
            idStringformat: Row id output text on the found error.
                            You can use the 'unicode.format' function notation {#} to transfer the row values.
        Returns:
            List containig list of two values - row id string and row error description string
        """
        rezultList = []
        with self.gp.da.SearchCursor(festureClass, field_names=fields, where_clause=where_clause) as cursor:
            for row in cursor:
                if not row[fields.index(getField)] in valuelist:
                    rezultList.append([unicode(idStringformat).format(*row), unicode(outStringformat).format(*row)])
        return rezultList

class RowHelper(RowHelper2):
    """Row processing class"""

    def __init__(self, gp, Tool=None):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            gp: ArcPy GP object
        """
        RowHelper2.__init__(self, gp, Tool)

    def ValidateRowsForSQLClause(self, festureClass, fields, where_clause, outStringformat):
        """Validate the rows with the SQL clause

        Args:
            self: The reserved object 'self'
            festureClass: The feature class containing the rows to be searched
            fields: A list of the field names (order is important, because the parameter 'outStringformat' is configured by this parameter)
            where_clause: SQL WHERE clause to obtain the data
            outStringformat: Error output text on the found error.
                                You can use the 'unicode.format' function notation {#} to transfer the row values.
                                For example - a field with the index 0 in the list of fields with 'outStringformat' parameter value: u"faulty feature OID: {0}" will return the string: u"faulty feature OID: 123", where 123 is the 'objectid' field value for found record.
        """
        rezultList = RowHelper2.ValidateRowsForSQLClause(self, festureClass, fields, where_clause, outStringformat)
        return [row[1] for row in rezultList]

    def ValidateRowsForFieldValueList(self, festureClass, getField, fields, valuelist, where_clause, outStringformat):
        """Check rows if the field matches the unique value list

        Args:
            self: The reserved object 'self'
            festureClass: The feature class containing the rows to be searched
            getField: Field to check
            fields: A list of field names (order is important, because parameter 'outStringformat' is configured by this parameter !!! 'getField' value should be in this list!!!
            valuelist: A list of values
            where_clause: SQL WHERE clause to obtain the data
            outStringformat: Error output text on the found error.
                                You can use the 'unicode.format' function notation {#} to transfer the row values.
                                For example - a field with the index 0 in the list of fields with 'outStringformat' parameter value: u"faulty feature OID: {0}" will return the string: u"faulty feature OID: 123", where 123 is the 'objectid' field value for found record.
        """
        rezultList = RowHelper2.ValidateRowsForFieldValueList(self, festureClass, getField, fields, valuelist, where_clause, outStringformat)
        return [row[1] for row in rezultList]

class SimpleAppend:
    """Class for easing the operations with the 'Append' function"""
    def __init__(self, _Tool, _InWSp, _OutWSp):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            self.InWS: Input workspace
            self.OutWSp: Output workspace
        """
        self.Tool = _Tool
        self.InWSp = _InWSp
        self.OutWSp = _OutWSp

    def Append(self, inName, OutName, do_data_delete = True):
        """Function for 'Append' operation automation

        Args:
            self: The reserved object 'self'
            inName: Input object name
            OutName:  Output object name
        """
        inLayer = os.path.join(self.InWSp, inName)
        outLayer = os.path.join(self.OutWSp, OutName)
        self.Tool.AddMessage("")
        self.Tool.AddMessage("-----------" + OutName + "------------" + self.Tool.MyNow())
        if do_data_delete:
            self.Tool.gp.DeleteRows_management(outLayer)
            self.Tool.OutputMessages()
        self.Tool.gp.Append_management(inLayer, outLayer, "NO_TEST", "#", "#")
        self.Tool.OutputMessages()
        try:
            self.Tool.gp.Analyze_management(outLayer, "BUSINESS")
            self.Tool.OutputMessages()
        except:
            self.Tool.AddMessage(u"Warning - could not carry out the 'Analyze' operation!")

def uni(value):
    if value == None:
        return ''
    try:
        return unicode(value)
    except:
        return value.decode('cp1257')



# -*- coding: utf-8 -*-
"""
     Module for the GISPython module frame and code unification
"""

import sys
import os
import traceback
import GISPythonToolBase
import SysGISTools
import MailHelper
import MyError

class GISPythonModule(object):
    """Interface class for all the GISPython modules. Interface class allows
    the code unification, and ensures the code execution from both the
    ArcGIS Desktop Python console and the Command Prompt.

    Standalone tool execution:
        SetName('Tool Name')
        DoJob()

    The tool executes within an another tool:
        SetTool(reference to SysGISTools.GISTools10)
        Get the tool name with the command 'PrintText()'
        mainModule()
    """
    def __init__(self, ToolName, SysGISParams, ExecutePatch=__file__, statusMailRecipients=[], errorMailRecipients=[], licenceLevel = "arceditor", toollevel = "full"):
        """Initialize the tool and the tool parameters

        Args:
            self: The reserved object 'self'
            ToolName: Tool name
            SysGISParams: GISPython parameter module
            ExecutePatch: Module execution path
            statusMailRecipients: List with status e-mail message recipients
            errorMailRecipients: List with error e-mail message recipients
            licenceLevel: arcinfo, arceditor, arcview, arcserver, arcenginegeodb, or arcengine
        """
        self.ToolName = ToolName
        self.licenceLevel = licenceLevel
        if toollevel.upper() in ("FULL", "LITE"):
            self.toollevel = toollevel.upper()
        else:
            raise AttributeError('Provided Tool Type {} is not supported!'.format(toollevel))
        self.Pr = SysGISParams
        self.ExecutePatch = os.path.dirname(os.path.realpath(ExecutePatch))
        if  hasattr(self.Pr, 'EnableStatusMail'):
            self.DoMail = self.Pr.EnableStatusMail
        else:
            self.DoMail = False
        if  hasattr(self.Pr, 'EnvironmentName'):
            self.EnvironmentName = '[{}] '.format(self.Pr.EnvironmentName)
        else:
            self.EnvironmentName = ''
        self.DoMailOutput = False
        self.DoMailErrOutput = False
        if self.DoMail:
            self.statusMailRecipients = statusMailRecipients
            if  hasattr(self.Pr, 'MailOutputRecipientsAlways'):
                self.statusMailRecipients = self.statusMailRecipients + self.Pr.MailOutputRecipientsAlways
            self.errorMailRecipients = errorMailRecipients
            if  hasattr(self.Pr, 'MailErrorRecipientsAlways'):
                self.errorMailRecipients = self.errorMailRecipients + self.Pr.MailErrorRecipientsAlways
            if self.statusMailRecipients != []:
                self.DoMailOutput = True
            if self.errorMailRecipients != []:
                self.DoMailErrOutput = True

    def runInsideJob(self, Tool):
        """Procedure executes the tool, if it's to be executed within an another Python tool

        Args:
            self: The reserved object 'self'
            Tool: The tool name to execute
        """
        self.SetTool(Tool)
        self.PrintText()
        self.mainModule()

    def mainModule(self):
        """Rewritable procedure which contains the logic of the module
        """
        raise NotImplementedError

    def initModule(self):
        """Procedure which initializes the GISPython environment, if the tool runs as a standalone tool
        """
        if self.toollevel == "FULL":
            self.Tool = SysGISTools.GISTools10(self.ToolName, self.Pr, self.licenceLevel)
            sys.stderr = self.Tool.fLog
        elif self.toollevel == "LITE":
            self.Tool = GISPythonToolBase.GISPythonToolBase(self.ToolName, self.Pr)
        self.Tool.ExecutePatch = self.ExecutePatch

    def SetTool(self, Tool):
        """Sets up the GISPython environment object, if the tool runs within an another tool
        """
        self.Tool = Tool

    def PrintText(self):
        """The auxiliary procedure for the tool name output
        """
        self.Tool.AddMessage("--------------------------------------------")
        self.Tool.AddMessage(self.ToolName)
        self.Tool.AddMessage("--------------------------------------------")

    def MyEnd(self):
        """Procedure for the tool end message output, if the tool runs as a standalone tool
        """
        self.Tool.MyEnd()

    def MyDispose(self):
        """Procedure which closes the tool environment after running it (in case the tool runs as a standalone tool)
        """
        self.Tool.MyDispose()

    def DoJob(self):
        """Procedure which runs the tool with environment preparation and deletion (in case the tool runs as a standalone tool)
        """
        try:
            self.initModule()
            self.mainModule()
            self.MyEnd()
            if self.DoMailOutput:
                MailHelper.GISPythonMailHelper(self.Pr, self.statusMailRecipients, unicode.format(u'{}[{}] - {}', self.EnvironmentName, self.ToolName, self.Tool.MyNow()), self.Tool.OutputStr)
            if self.DoMailErrOutput:
                if not self.Tool.OutputErrStr == u'':
                    MailHelper.GISPythonMailHelper(self.Pr, self.errorMailRecipients, unicode.format(u'{}Error [{}] - {}', self.EnvironmentName, self.ToolName, self.Tool.MyNow()), self.Tool.OutputErrStr)
            self.MyDispose()
        except Exception, e:
            # If an error occurred, print line number and error message
            tb = sys.exc_info()
            orgLine = "Line %i" % tb[2].tb_lineno
            orgTraceback = unicode(traceback.format_exc(), errors='ignore')
            if hasattr(self, 'Tool'):
                try:
                    if not (self.Tool.State == "Disposed" or self.Tool.gp == None):
                        self.Tool.AddWarning(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                        try:
                            self.Tool.AddWarning(u'Raw tool error {0} - {1}'.format(self.ToolName, self.Tool.MyNow()))
                        except:
                            None
                        self.Tool.AddWarning(orgLine)
                        self.Tool.AddError(orgTraceback)
                        self.Tool.AddWarning(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                        self.Tool.AddWarning(u'GP tool output')
                        self.Tool.AddWarning(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                        self.Tool.OutputErrors()
                        if self.DoMailErrOutput:
                            if self.Tool.OutputErrStr != u'':
                                MailHelper.GISPythonMailHelper(self.Pr, self.errorMailRecipients, unicode.format(u'{}Error [{}] - {}', self.EnvironmentName, self.ToolName, self.Tool.MyNow()), self.Tool.OutputErrStr)
                    else:
                        print(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                        print(u'Raw tool error {0}'.format(self.ToolName))
                        print(orgLine)
                        print(orgTraceback)
                        print(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                        if self.DoMailErrOutput:
                            MailHelper.GISPythonMailHelper(self.Pr, self.errorMailRecipients, unicode.format(u'{}Critical error  [{}]', self.EnvironmentName, self.ToolName), orgLine + '\n' + orgTraceback)
                        raise
                except Exception:
                    print(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                    print(u'Raw tool error {0}'.format(self.ToolName))
                    print(orgLine)
                    print(orgTraceback)
                    print(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                    if self.DoMailErrOutput:
                        MailHelper.GISPythonMailHelper(self.Pr, self.errorMailRecipients, unicode.format(u'{}Critical error [{}]', self.EnvironmentName, self.ToolName), orgLine + '\n' + orgTraceback)
                    raise tb[1], None, tb[2]
            else:
                print(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                print(u'Raw tool error {0}'.format(self.ToolName))
                print(orgLine)
                print(orgTraceback)
                print(r'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=')
                if self.DoMailErrOutput:
                    MailHelper.GISPythonMailHelper(self.Pr, self.errorMailRecipients, unicode.format(u'{}Critical error [{}]', self.EnvironmentName, self.ToolName), orgLine + '\n' + orgTraceback)
                raise

class GISPythonModuleArgsHelper(object):
    """Class for handling the argument passing for the module in three different places:

    1. In the class initialization process.
    2. In the arguments.
    3. In the "main" operation call.
    """
    def __init__(self, InitValue=None):
        self.argumentValue = None
        self.initValue = None
        self.mainModuleValue = None
        self.SetInitValue(InitValue)

    def processArgument(self, argumentNumber=1):
        """Procedure processes a parameter acquisition from the argument

        Args:
            self: The reserved object 'self'
            argumentNumber: Argument from which to read the data
        """
        if len(sys.argv) > argumentNumber:
            self.argumentValue = sys.argv[argumentNumber]
            if self.argumentValue == '#':
                self.argumentValue = None


    def SetInitValue(self, value):
        """Procedure processes a parameter setup from the tool initialization procedure

        Args:
            self: The reserved object 'self'
            value: Setup value
        """
        self.initValue = value
        if self.initValue == '#':
            self.initValue = None

    def SetMainModuleValue(self, value):
        """Procedure processes a parameter setup from the base module of the tool

        Args:
            self: The reserved object 'self'
            value: Setup value
        """
        self.mainModuleValue = value
        if self.mainModuleValue == '#':
            self.mainModuleValue = None

    def GetResultValue(self, asBool=False, Default=None):
        """Procedure makes a choice from the given attributes

        Args:
            self: The reserved object self
            value: Setup value
        """
        candidate = Default

        if self.argumentValue != None:
            candidate = self.argumentValue
        else:
            if self.mainModuleValue != None:
                candidate = self.mainModuleValue
            else:
                if self.initValue != None:
                    candidate = self.initValue
        if asBool:
            if candidate == None:
                candidate = False
            elif unicode(candidate).upper() == u'TRUE':
                candidate = True
            elif unicode(candidate).upper() == u'FALSE':
                candidate = False
            else:
                raise MyError.MyError(u'Cannot convert the value {0} to a boolean'.format(candidate))

        return candidate



# -*- coding: utf-8 -*-
"""
     FTP operations module
"""

import datetime
import os
import traceback
import sys
import MyError

class FTPFile:
    """Class for describing the FTP file"""

    def __init__(self, file, date, size):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            file: Filename
            date: File date
            size: File size
        """
        self.file = file
        self.date = date
        self.size = size

class FTPHleper:
    """Class for easing the FTP operations"""

    def __init__(self, FTPHost, FTPUser, FTPPwd, FTPDir=None):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            FTPHost: FTP server
            FTPUser: FTP username
            FTPPwd: FTP user password
        """
        import ftplib

        try:
            self.ftp = ftplib.FTP(FTPHost, FTPUser, FTPPwd)
            if FTPDir == None:
                self.ftp.cwd('/')
            else:
                self.ftp.cwd(FTPDir)
        except Exception, e:

            tb = sys.exc_info()
            txt = "Line %i" % tb[2].tb_lineno + unicode(traceback.format_exc(), errors='ignore')
            raise MyError.MyError(u"Error connecting to ftp: " + FTPHost + u'\n Error output: \n\n' + txt)


    def list_files(self):
        """Procedure to retrieve a file description in the specific connection directory

        Args:
            self: The reserved object 'self'
        """
        self.files = []
        self.ftp.dir(self.dir_callback)
        return self.files

    def get_file_date(self, fileName):
        """Determines the ftp file modification date

        Args:
            self: The reserved object 'self'
            fileName: Ftp file name
        """
        modifiedTime = self.ftp.sendcmd('MDTM ' + fileName)
        modifiedTime = self.ftp.sendcmd('MDTM ' + fileName)
        return datetime.datetime.strptime(modifiedTime[4:], "%Y%m%d%H%M%S")

    def get_file_size(self, fileName):
        """Determines the ftp file size

        Args:
            self: The reserved object 'self'
            fileName: Ftp file name
        """
        size = self.ftp.sendcmd('SIZE ' + fileName)
        size = self.ftp.sendcmd('SIZE ' + fileName)
        return int(size[4:])

    def get_file(self, filename, savePath):
        """Retrieves the file from the FTP server

        Args:
            self: The reserved object 'self'
            filename: Ftp file name
        """
        # Open the file for writing in binary mode
        print 'Opening local file ' + savePath
        file = open(savePath, 'wb')
        fSize = [0, 0]

        # Download the file a chunk at a time
        # Each chunk is sent to handleDownload
        # We append the chunk to the file and then print a '.' for progress
        # RETR is an FTP command

        def download_file_callback(data):
            """Download file callback"""
            file.write(data)
            fSize[0] += len(data)
            fSize[1] += len(data)
            if fSize[1] >= 10485760:
                file.flush()
                print 'Downloaded ' + str(fSize[0]/1048576) + ' MB'
                fSize[1] = 0

        print 'Getting ' + savePath
        self.ftp.retrbinary('RETR %s' % filename, download_file_callback, 4096)

        # Clean up time
        print 'Closing file ' + savePath
        file.close()

    def upload_file(self, fileName, filePath):
        """Uploads the binary file, using FTP

        Args:
            self: The reserved object 'self'
            filePath: Uploadable file local path
            fileName: Uploadable file name
        """
        uploadFile = open(os.path.join(filePath, fileName), 'rb')
        # Store a file in binary transfer mode
        self.ftp.storbinary('STOR ' + fileName, uploadFile)
        # close file
        uploadFile.close()

    def delete_file(self, fileName):
        """Deletes the file from the FTP server

        Args:
            self: The reserved object 'self'
            fileName: Name of the file to delete
        """
        self.ftp.delete(fileName)

    def dir_callback(self, line):
        """Processes callback from the procedure 'list_files'

        Args:
            self: The reserved object 'self'
            line: Row with the FTP file description
        """
        bits = line.split()

        if 'd' not in bits[0]:
            fFile = FTPFile(bits[-1], self.get_file_date(bits[-1]), self.get_file_size(bits[-1]))
            self.files.append(fFile)



# -*- coding: utf-8 -*-
"""
     Timing module
"""

import datetime

class TimerHelper(object):
    """Class for easing the countdown procedures"""

    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """
        self.StartTime = datetime.datetime.now()

    def TimerReset(self):
        """Reset the countdown

        Args:
            self: The reserved object 'self'
        """
        self.StartTime = datetime.datetime.now()


    def GetTime(self):
        """Get the elapsed time

        Args:
            self: The reserved object 'self'

        Returns:
            Output text as a string
        """
        TD = datetime.datetime.now() - self.StartTime
        return str(TD)

    def GetTimeReset(self):
        """Reset the elapsed time

        Args:
            self: The reserved object 'self'

        Returns:
            Output text as a string
        """
        TD = self.GetTime()
        self.TimerReset()
        return TD

class TimedSubprocess:
    """Class for providing a code block with timing capabilities"""

    def __init__(self, Tool, txt, lvl=1):
        """Class initialization procedure
        Args:
            self: The reserved object 'self'
            Tool: Reference to the GISTools10 object
            txt: Output text
            lvl: Sublevel
        """
        self.StartTime = datetime.datetime.now()
        self.Tool = Tool
        self.txt = txt
        self.T = TimerHelper()
        self.prefix = ""
        for i in range(0, lvl-1):
            self.prefix += '    '
    def __enter__(self):
        """With statement opening procedure
        Args:
            self: The reserved object 'self'
        """
        self.Tool.AddMessage(u'\n\n{0}------------------------\n{0}>>>> Begin {1} - {2}\n{0}------------------------\n'.format(self.prefix, self.txt, self.Tool.MyNow()))
        return self
    def __exit__(self, type, value, traceback):
        """With statement closing procedure
        Args:
            self: The reserved object 'self'
        """
        self.Tool.AddMessage(u'\n{0}------------------------\n{0}>>>> End {1} - {2}\n{0}------------------------\n'.format(self.prefix, self.txt, self.T.GetTimeReset()))
